;
; Definition file of wlanui.dll
; Automatic generated by gendef
; written by Kai Tietz 2008
;
LIBRARY "wlanui.dll"
EXPORTS
WLInvokeProfileUI@32
WLInvokeProfileUIFromXMLFile@40
DllGetClassObject@12
WLFreeProfile@4
WLFreeProfileXml@4
WlanUIEditProfile@28
