; 
; Exports of file imeskdic.dll
;
; Autogenerated by gen_exportdef
; Written by Kai Tietz, 2007
;
LIBRARY imeskdic.dll
EXPORTS
CreateIImeSkdicInstance
DllCanUnloadNow
DllGetClassObject
DllRegisterServer
DllUnregisterServer
