; 
; Exports of file ICM32.dll
;
; Autogenerated by gen_exportdef
; Written by Kai Tietz, 2007
;
LIBRARY ICM32.dll
EXPORTS
CMCheckColors
CMCheckColorsInGamut
CMCheckRGBs
CMCreateDeviceLinkProfile
CMCreateMultiProfileTransform
CMCreateProfile
CMCreateProfileW
CMCreateTransform
CMCreateTransformExt
CMCreateTransformExtW
CMCreateTransformW
CMDeleteTransform
CMGetInfo
CMIsProfileValid
CMTranslateColors
CMTranslateRGB
CMTranslateRGBs
CMTranslateRGBsExt
CMConvertColorNameToIndex
CMConvertIndexToColorName
CMGetNamedProfileInfo
