;
; Definition file of AVRT.dll
; Automatic generated by gendef
; written by Kai Tietz 2008
;
LIBRARY "AVRT.dll"
EXPORTS
AvQuerySystemResponsiveness@8
AvRevertMmThreadCharacteristics@4
AvRtCreateThreadOrderingGroup@16
AvRtCreateThreadOrderingGroupExA@20
AvRtCreateThreadOrderingGroupExW@20
AvRtDeleteThreadOrderingGroup@4
AvRtJoinThreadOrderingGroup@12
AvRtLeaveThreadOrderingGroup@4
AvRtWaitOnThreadOrderingGroup@4
AvSetMmMaxThreadCharacteristicsA@12
AvSetMmMaxThreadCharacteristicsW@12
AvSetMmThreadCharacteristicsA@8
AvSetMmThreadCharacteristicsW@8
AvSetMmThreadPriority@8
