;
; Definition file of PROFEXT.dll
; Automatic generated by gendef
; written by Kai Tietz 2008-2014
;
LIBRARY "PROFEXT.dll"
EXPORTS
CreateAppContainerProfileWorker
CreateDirectoryJunctionsForSystemWorker
CreateDirectoryJunctionsForUserProfileWorker
CreateGroupExWorker
CreateLinkFileExWorker
DeleteAppContainerProfileWorker
DeleteGroupWorker
DeleteLinkFileWorker
DeriveAppContainerSidFromAppContainerNameWorker
DeriveRestrictedAppContainerSidFromAppContainerSidAndRestrictedNameWorker
GetAppContainerFolderPathWorker
GetAppContainerRegistryLocationWorker
LookupAppContainerDisplayNameWorker
ProcessGroupPolicyCompletedExWorker
ProcessGroupPolicyCompletedWorker
RsopAccessCheckByTypeWorker
RsopFileAccessCheckWorker
RsopResetPolicySettingStatusWorker
RsopSetPolicySettingStatusWorker
UpdateAppContainerProfileWorker
