;
; Definition file of WININITEXT.dll
; Automatic generated by gendef
; written by Kai Tietz 2008-2014
;
LIBRARY "WININITEXT.dll"
EXPORTS
GetLoggedOnUserCount
PrimaryTerminalAndHookWorker
StartLoadingFontsWorker
UIStartupWorker
UnregisterSession0ViewerWindowHookDll
WaitForWinstationShutdown
WinStationSystemShutdownStartedWorker
