;
; Definition file of SPFILEQ.dll
; Automatic generated by gendef
; written by Kai Tietz 2008-2014
;
LIBRARY "SPFILEQ.dll"
EXPORTS
SpFileQueueClose
SpFileQueueCommit
SpFileQueueCopy
SpFileQueueDelete
SpFileQueueFileInUse
SpFileQueueGetFlags
SpFileQueueGetQueueCount
SpFileQueueNodeGetSecurityDescriptor
SpFileQueueNodeGetSourceFilename
SpFileQueueNodeGetSourcePath
SpFileQueueNodeGetSourceRootPath
SpFileQueueNodeGetStyleFlags
SpFileQueueNodeGetTargetDirectory
SpFileQueueNodeGetTargetFilename
SpFileQueueNodeRemove
SpFileQueueOpen
SpFileQueueRename
SpFileQueueSetFlags
