;
; Definition file of SXSHARED.dll
; Automatic generated by gendef
; written by Kai Tietz 2008-2014
;
LIBRARY "SXSHARED.dll"
EXPORTS
GetLastFailureAsHRESULT
HRESULTFromNTSTATUS
SxTracerDebuggerBreak
SxTracerGetThreadContextDebug
SxTracerGetThreadContextRetail
SxTracerShouldTrackFailure
Win32FromHRESULT
Win32FromNTSTATUS
