;
; Definition file of P2PGRAPH.dll
; Automatic generated by gendef
; written by Kai Tietz 2008
;
LIBRARY "P2PGRAPH.dll"
EXPORTS
pMemoryHelper DATA
PeerGraphAddRecord@12
PeerGraphClose@4
PeerGraphCloseDirectConnection@12
PeerGraphConnect@16
PeerGraphCreate@16
PeerGraphDelete@12
PeerGraphDeleteRecord@12
PeerGraphEndEnumeration@4
PeerGraphEnumConnections@12
PeerGraphEnumNodes@12
PeerGraphEnumRecords@16
PeerGraphExportDatabase@8
PeerGraphFreeData@4
PeerGraphGetEventData@8
PeerGraphGetItemCount@8
PeerGraphGetNextItem@12
PeerGraphGetNodeInfo@16
PeerGraphGetProperties@8
PeerGraphGetRecord@12
PeerGraphGetStatus@8
PeerGraphImportDatabase@8
PeerGraphListen@16
PeerGraphOpen@28
PeerGraphOpenDirectConnection@16
PeerGraphPeerTimeToUniversalTime@12
PeerGraphRegisterEvent@20
PeerGraphSearchRecords@12
PeerGraphSendData@24
PeerGraphSetNodeAttributes@8
PeerGraphSetPresence@8
PeerGraphSetProperties@8
PeerGraphShutdown@0
PeerGraphStartup@8
PeerGraphUniversalTimeToPeerTime@12
PeerGraphUnregisterEvent@4
PeerGraphUpdateRecord@8
PeerGraphValidateDeferredRecords@12
