;
; Definition file of NETAPI32.dll
; Automatic generated by gendef
; written by Kai Tietz 2008
;
LIBRARY "NETAPI32.dll"
EXPORTS
CredpValidateTargetName@52
DavAddConnection@24
DavDeleteConnection@4
DavFlushFile@4
DavGetExtendedError@16
DavGetHTTPFromUNCPath@12
DavGetUNCFromHTTPPath@12
DsAddressToSiteNamesA@16
DsAddressToSiteNamesExA@20
DsAddressToSiteNamesExW@20
DsAddressToSiteNamesW@16
DsDeregisterDnsHostRecordsA@20
DsDeregisterDnsHostRecordsW@20
DsEnumerateDomainTrustsA@16
DsEnumerateDomainTrustsW@16
DsGetDcCloseW@4
DsGetDcNameA@24
DsGetDcNameW@24
DsGetDcNameWithAccountA@32
DsGetDcNameWithAccountW@32
DsGetDcNextA@16
DsGetDcNextW@16
DsGetDcOpenA@28
DsGetDcOpenW@28
DsGetDcSiteCoverageA@12
DsGetDcSiteCoverageW@12
DsGetForestTrustInformationW@16
DsGetSiteNameA@8
DsGetSiteNameW@8
DsMergeForestTrustInformationW@16
DsRoleAbortDownlevelServerUpgrade@16
DsRoleCancel@8
DsRoleDcAsDc@68
DsRoleDcAsReplica@12
DsRoleDemoteDc@44
DsRoleDnsNameToFlatName@16
DsRoleFreeMemory@4
DsRoleGetDatabaseFacts@24
DsRoleGetDcOperationProgress@12
DsRoleGetDcOperationResults@12
DsRoleGetPrimaryDomainInformation@12
DsRoleIfmHandleFree@8
DsRoleServerSaveStateForUpgrade@4
DsRoleUpgradeDownlevelServer@48
DsValidateSubnetNameA@4
DsValidateSubnetNameW@4
I_BrowserDebugCall@12
I_BrowserDebugTrace@8
I_BrowserQueryEmulatedDomains@12
I_BrowserQueryOtherDomains@16
I_BrowserQueryStatistics@8
I_BrowserResetNetlogonState@4
I_BrowserResetStatistics@4
I_BrowserServerEnum@44
I_BrowserSetNetlogonState@16
I_DsUpdateReadOnlyServerDnsRecords@28
I_NetAccountDeltas@48
I_NetAccountSync@48
I_NetChainSetClientAttributes2@36
I_NetChainSetClientAttributes@36
I_NetDatabaseDeltas@32
I_NetDatabaseRedo@28
I_NetDatabaseSync2@36
I_NetDatabaseSync@32
I_NetDfsCreateExitPoint@24
I_NetDfsCreateLocalPartition@28
I_NetDfsDeleteExitPoint@16
I_NetDfsDeleteLocalPartition@12
I_NetDfsFixLocalVolume@36
I_NetDfsGetFtServers@16
I_NetDfsGetVersion@8
I_NetDfsIsThisADomainName@4
I_NetDfsManagerReportSiteInfo@8
I_NetDfsModifyPrefix@12
I_NetDfsSetLocalVolumeState@16
I_NetDfsSetServerInfo@12
I_NetGetDCList@16
I_NetGetForestTrustInformation@24
I_NetListCanonicalize@36
I_NetListTraverse@12
I_NetLogonControl2@20
I_NetLogonControl@16
I_NetLogonGetDomainInfo@28
I_NetLogonSamLogoff@24
I_NetLogonSamLogon@36
I_NetLogonSamLogonEx@40
I_NetLogonSamLogonWithFlags@40
I_NetLogonSendToSam@24
I_NetLogonUasLogoff@12
I_NetLogonUasLogon@12
I_NetNameCanonicalize@24
I_NetNameCompare@20
I_NetNameValidate@16
I_NetPathCanonicalize@28
I_NetPathCompare@20
I_NetPathType@16
I_NetServerAuthenticate2@28
I_NetServerAuthenticate3@32
I_NetServerAuthenticate@24
I_NetServerGetTrustInfo@36
I_NetServerPasswordGet@28
I_NetServerPasswordSet2@28
I_NetServerPasswordSet@28
I_NetServerReqChallenge@16
I_NetServerSetServiceBits@16
I_NetServerSetServiceBitsEx@24
I_NetServerTrustPasswordsGet@32
I_NetlogonComputeClientDigest@24
I_NetlogonComputeServerDigest@24
I_NetlogonGetTrustRid@12
NetAccessAdd@16
NetAccessDel@8
NetAccessEnum@36
NetAccessGetInfo@16
NetAccessGetUserPerms@16
NetAccessSetInfo@20
NetAddAlternateComputerName@20
NetAddServiceAccount@16
NetAlertRaise@12
NetAlertRaiseEx@16
NetApiBufferAllocate@8
NetApiBufferFree@4
NetApiBufferReallocate@12
NetApiBufferSize@8
NetAuditClear@12
NetAuditRead@44
NetAuditWrite@20
NetBrowserStatisticsGet@12
NetConfigGet@16
NetConfigGetAll@12
NetConfigSet@28
NetConnectionEnum@32
NetDfsAdd@20
NetDfsAddFtRoot@20
NetDfsAddRootTarget@20
NetDfsAddStdRoot@16
NetDfsAddStdRootForced@16
NetDfsEnum@24
NetDfsGetClientInfo@20
NetDfsGetDcAddress@16
NetDfsGetFtContainerSecurity@16
NetDfsGetInfo@20
NetDfsGetSecurity@16
NetDfsGetStdContainerSecurity@16
NetDfsGetSupportedNamespaceVersion@12
NetDfsManagerGetConfigInfo@28
NetDfsManagerInitialize@8
NetDfsManagerSendSiteInfo@12
NetDfsMove@12
NetDfsRemove@12
NetDfsRemoveFtRoot@16
NetDfsRemoveFtRootForced@20
NetDfsRemoveRootTarget@12
NetDfsRemoveStdRoot@12
NetDfsRename@8
NetDfsSetClientInfo@20
NetDfsSetFtContainerSecurity@12
NetDfsSetInfo@20
NetDfsSetSecurity@12
NetDfsSetStdContainerSecurity@12
NetEnumerateComputerNames@20
NetEnumerateServiceAccounts@16
NetEnumerateTrustedDomains@8
NetErrorLogClear@12
NetErrorLogRead@44
NetErrorLogWrite@32
NetFileClose@8
NetFileEnum@36
NetFileGetInfo@16
NetGetAnyDCName@12
NetGetDCName@12
NetGetDisplayInformationIndex@16
NetGetJoinInformation@12
NetGetJoinableOUs@24
NetGroupAdd@16
NetGroupAddUser@12
NetGroupDel@8
NetGroupDelUser@12
NetGroupEnum@28
NetGroupGetInfo@16
NetGroupGetUsers@32
NetGroupSetInfo@20
NetGroupSetUsers@20
NetIsServiceAccount@12
NetJoinDomain@24
NetLocalGroupAdd@16
NetLocalGroupAddMember@12
NetLocalGroupAddMembers@20
NetLocalGroupDel@8
NetLocalGroupDelMember@12
NetLocalGroupDelMembers@20
NetLocalGroupEnum@28
NetLocalGroupGetInfo@16
NetLocalGroupGetMembers@32
NetLocalGroupSetInfo@20
NetLocalGroupSetMembers@20
NetLogonGetTimeServiceParentDomain@12
NetLogonSetServiceBits@12
NetMessageBufferSend@20
NetMessageNameAdd@8
NetMessageNameDel@8
NetMessageNameEnum@28
NetMessageNameGetInfo@16
NetProvisionComputerAccount@32
NetQueryDisplayInformation@28
NetQueryServiceAccount@16
NetRegisterDomainNameChangeNotification@4
NetRemoteComputerSupports@12
NetRemoteTOD@8
NetRemoveAlternateComputerName@20
NetRemoveServiceAccount@12
NetRenameMachineInDomain@20
NetReplExportDirAdd@16
NetReplExportDirDel@8
NetReplExportDirEnum@28
NetReplExportDirGetInfo@16
NetReplExportDirLock@8
NetReplExportDirSetInfo@20
NetReplExportDirUnlock@12
NetReplGetInfo@12
NetReplImportDirAdd@16
NetReplImportDirDel@8
NetReplImportDirEnum@28
NetReplImportDirGetInfo@16
NetReplImportDirLock@8
NetReplImportDirUnlock@12
NetReplSetInfo@16
NetRplAdapterAdd@16
NetRplAdapterDel@8
NetRplAdapterEnum@28
NetRplBootAdd@16
NetRplBootDel@12
NetRplBootEnum@28
NetRplClose@4
NetRplConfigAdd@16
NetRplConfigDel@8
NetRplConfigEnum@32
NetRplGetInfo@12
NetRplOpen@8
NetRplProfileAdd@16
NetRplProfileClone@16
NetRplProfileDel@8
NetRplProfileEnum@32
NetRplProfileGetInfo@16
NetRplProfileSetInfo@20
NetRplSetInfo@16
NetRplSetSecurity@16
NetRplVendorAdd@16
NetRplVendorDel@8
NetRplVendorEnum@28
NetRplWkstaAdd@16
NetRplWkstaClone@24
NetRplWkstaDel@8
NetRplWkstaEnum@32
NetRplWkstaGetInfo@16
NetRplWkstaSetInfo@20
NetRequestOfflineDomainJoin@16
NetScheduleJobAdd@12
NetScheduleJobDel@12
NetScheduleJobEnum@24
NetScheduleJobGetInfo@12
NetServerAliasAdd@12
NetServerAliasDel@12
NetServerAliasEnum@28
NetServerComputerNameAdd@12
NetServerComputerNameDel@8
NetServerDiskEnum@28
NetServerEnum@36
NetServerEnumEx@36
NetServerGetInfo@12
NetServerSetInfo@16
NetServerTransportAdd@12
NetServerTransportAddEx@12
NetServerTransportDel@12
NetServerTransportEnum@28
NetServiceControl@20
NetServiceEnum@28
NetServiceGetInfo@16
NetServiceInstall@20
NetSessionDel@12
NetSessionEnum@36
NetSessionGetInfo@20
NetSetPrimaryComputerName@20
NetShareAdd@16
NetShareCheck@12
NetShareDel@12
NetShareDelEx@12
NetShareDelSticky@12
NetShareEnum@28
NetShareEnumSticky@28
NetShareGetInfo@16
NetShareSetInfo@20
NetStatisticsGet@20
NetUnjoinDomain@16
NetUnregisterDomainNameChangeNotification@4
NetUseAdd@16
NetUseDel@12
NetUseEnum@28
NetUseGetInfo@16
NetUserAdd@16
NetUserChangePassword@16
NetUserDel@8
NetUserEnum@32
NetUserGetGroups@28
NetUserGetInfo@16
NetUserGetLocalGroups@32
NetUserModalsGet@12
NetUserModalsSet@16
NetUserSetGroups@20
NetUserSetInfo@20
NetValidateName@20
NetValidatePasswordPolicy@20
NetValidatePasswordPolicyFree@4
NetWkstaGetInfo@12
NetWkstaSetInfo@16
NetWkstaTransportAdd@16
NetWkstaTransportDel@12
NetWkstaTransportEnum@28
NetWkstaUserEnum@28
NetWkstaUserGetInfo@12
NetWkstaUserSetInfo@16
NetapipBufferAllocate@8
Netbios@4
NetpAccessCheck@12
NetpAccessCheckAndAudit@20
NetpAccessCheckAndAuditEx@28
NetpAddTlnFtinfoEntry@8
NetpAllocConfigName@16
NetpAllocFtinfoEntry@20
NetpAllocStrFromWStr@4
NetpAllocWStrFromStr@4
NetpAllocWStrFromWStr@4
NetpApiStatusToNtStatus@4
NetpAssertFailed@16
NetpCleanFtinfoContext@4
NetpCloseConfigData@4
NetpCopyFtinfoContext@4
NetpCopyStringToBuffer@20
NetpCreateSecurityObject@24
NetpDbgDisplayServerInfo@8
NetpDbgPrint@0
NetpDeleteSecurityObject@4
NetpEventlogClose@4
NetpEventlogOpen@8
NetpEventlogWriteEx@32
NetpGetComputerName@4
NetpGetConfigBool@16
NetpGetConfigDword@16
NetpGetConfigTStrArray@12
NetpGetConfigValue@12
NetpGetDomainName@4
NetpGetFileSecurity@16
NetpGetPrivilege@8
NetpHexDump@8
NetpInitOemString@8
NetpInitFtinfoContext@4
NetpIsRemote@20
NetpIsUncComputerNameValid@4
NetpLocalTimeZoneOffset@0
NetpLogonPutUnicodeString@12
NetpNetBiosAddName@12
NetpNetBiosCall@16
NetpNetBiosDelName@8
NetpNetBiosGetAdapterNumbers@8
NetpNetBiosHangup@8
NetpNetBiosReceive@24
NetpMergeFtinfo@16
NetpNetBiosReset@4
NetpNetBiosSend@16
NetpNetBiosStatusToApiStatus@4
NetpNtStatusToApiStatus@4
NetpOpenConfigData@16
NetpPackString@12
NetpReleasePrivilege@0
NetpSetConfigBool@12
NetpSetConfigDword@12
NetpSetConfigTStrArray@12
NetpParmsQueryUserProperty@16
NetpParmsQueryUserPropertyWithLength@16
NetpParmsSetUserProperty@28
NetpParmsUserPropertyFree@4
NetpOpenConfigData@12
NetpSetFileSecurity@12
NetpSmbCheck@20
NetpStringToNetBiosName@16
NetpStoreIntialDcRecord@4
NetpTStrArrayEntryCount@4
NetpUpgradePreNT5JoinInfo@0
NetpwNameCanonicalize@20
NetpwNameCompare@16
NetpwNameValidate@12
NetpwPathCanonicalize@24
NetpwPathCompare@16
NetpwPathType@12
NlBindingAddServerToCache@8
NlBindingRemoveServerFromCache@8
NlBindingSetAuthInfo@20
RxNetAccessAdd@16
RxNetAccessDel@8
RxNetAccessEnum@36
RxNetAccessGetInfo@16
RxNetAccessGetUserPerms@16
RxNetAccessSetInfo@20
RxNetServerEnum@40
RxNetUserPasswordSet@16
RxRemoteApi
