;
; Definition file of VirtDisk.dll
; Automatic generated by gendef
; written by Kai Tietz 2008
;
LIBRARY "VirtDisk.dll"
EXPORTS
AddVirtualDiskParent@8
ApplySnapshotVhdSet@12
AttachVirtualDisk@24
BreakMirrorVirtualDisk@4
CompactVirtualDisk@16
CreateVirtualDisk@36
DeleteSnapshotVhdSet@12
DeleteVirtualDiskMetadata@8
DetachVirtualDisk@12
EnumerateVirtualDiskMetadata@12
ExpandVirtualDisk@16
GetAllAttachedVirtualDiskPhysicalPaths@8
GetStorageDependencyInformation@20
GetVirtualDiskInformation@16
GetVirtualDiskMetadata@16
GetVirtualDiskOperationProgress@12
GetVirtualDiskPhysicalPath@12
MergeVirtualDisk@16
MirrorVirtualDisk@16
ModifyVhdSet@12
OpenVirtualDisk@24
QueryChangesVirtualDisk@40
ResizeVirtualDisk@16
SetVirtualDiskInformation@8
SetVirtualDiskMetadata@16
TakeSnapshotVhdSet@12
