; 
; Exports of file NCXP.dll
;
; Autogenerated by gen_exportdef
; Written by Kai Tietz, 2007
;
LIBRARY NCXP.dll
EXPORTS
EnableAutodial
GetDefaultDialupConnection
IsAutodialEnabled
SetDefaultDialupConnection
TestRunDll
DisableUserLevelAccessControl
EnumMatchingNetBindings
EnumNetAdapters
HrEnableDhcp
HrFromLastWin32Error
HrWideCharToMultiByte
InstallMSClient
InstallSharing
InstallTCPIP
IsAccessControlUserLevel
IsAdapterDisconnected
IsClientInstalled
IsMSClientInstalled
IsProtocolInstalled
IsSharingInstalled
NetConnAlloc
NetConnFree
RestartNetAdapter
