;
; Definition file of apprepapi.dll
; Automatic generated by gendef
; written by Kai Tietz 2008-2014
;
LIBRARY "apprepapi.dll"
EXPORTS
AppRepComputeImageHash
AppRepComputeSignatureInfo
AppRepFreeAttributeLib
AppRepInitializeAttributeLib
AppRepParameterCleanup
RepGetFileInformation
RepGetFileReputation
RepInformUserAction
