LIBRARY "prntvpt.dll"
EXPORTS
PTQuerySchemaVersionSupport@8
PTOpenProvider@12
PTOpenProviderEx@20
PTCloseProvider@4
BindPTProviderThunk@20
PTGetPrintCapabilities@16
PTMergeAndValidatePrintTicket@24
PTConvertPrintTicketToDevMode@28
PTConvertDevModeToPrintTicket@20
PTReleaseMemory@4
PTGetPrintDeviceCapabilities@16
PTGetPrintDeviceResources@20
ConvertDevModeToPrintTicketThunk2@24
ConvertDevModeToPrintTicketThunk@20
ConvertPrintTicketToDevModeThunk2@32
ConvertPrintTicketToDevModeThunk@28
DllCanUnloadNow@0
DllGetClassObject@12
DllRegisterServer@0
DllUnregisterServer@0
GetDeviceDefaultPrintTicketThunk@12
GetDeviceNamespacesThunk@12
GetPrintCapabilitiesThunk2@24
GetPrintCapabilitiesThunk@20
GetPrintDeviceCapabilitiesThunk2@24
GetPrintDeviceCapabilitiesThunk@20
GetPrintDeviceResourcesThunk2@28
GetPrintDeviceResourcesThunk@24
GetSchemaVersionThunk@4
MergeAndValidatePrintTicketThunk2@36
MergeAndValidatePrintTicketThunk@28
UnbindPTProviderThunk@4
