; 
; Exports of file RICHED20.dll
;
; Autogenerated by gen_exportdef
; Written by Kai Tietz, 2007
;
LIBRARY RICHED20.dll
EXPORTS
IID_IRichEditOle
IID_IRichEditOleCallback
CreateTextServices
IID_ITextServices
IID_ITextHost
IID_ITextHost2
REExtendedRegisterClass
RichEdit10ANSIWndProc
RichEditANSIWndProc
