;
; Definition file of DNSAPI.dll
; Automatic generated by gendef
; written by Kai Tietz 2008
;
LIBRARY "DNSAPI.dll"
EXPORTS
DnsGetDomainName
DnsIsAMailboxType
DnsIsNSECType
DnsIsStatusRcode
DnsMapRcodeToStatus
DnsStatusString
DnsUnicodeToUtf8@8
DnsUtf8ToUnicode@8
Dns_ReadPacketName@20
Dns_ReadPacketNameAllocate@20
Dns_SkipPacketName
Dns_WriteDottedNameToPacket@16
AdaptiveTimeout_ClearInterfaceSpecificConfiguration
AdaptiveTimeout_ResetAdaptiveTimeout
AddRefQueryBlobEx@16
BreakRecordsIntoBlob@12
Coalesce_UpdateNetVersion
CombineRecordsInBlob@8
DeRefQueryBlobEx@16
DelaySortDAServerlist
DnsAcquireContextHandle_A@12
DnsAcquireContextHandle_W@12
DnsAllocateRecord@4
DnsApiAlloc@4
DnsApiAllocZero@4
DnsApiFree@4
DnsApiHeapReset@12
DnsApiRealloc@8
DnsApiSetDebugGlobals@4
DnsAsyncRegisterHostAddrs@40
DnsAsyncRegisterInit@4
DnsAsyncRegisterTerm
DnsCancelQuery@4
DnsCheckNrptRuleIntegrity@4
DnsCheckNrptRules@12
DnsCleanupTcpConnections@4
DnsConnectionDeletePolicyEntries@4
DnsConnectionDeletePolicyEntriesPrivate@8
DnsConnectionDeleteProxyInfo@8
DnsConnectionFreeNameList@4
DnsConnectionFreeProxyInfo@4
DnsConnectionFreeProxyInfoEx@4
DnsConnectionFreeProxyList@4
DnsConnectionGetHandleForHostUrlPrivate@24
DnsConnectionGetNameList@4
DnsConnectionGetProxyInfo@12
DnsConnectionGetProxyInfoForHostUrl@20
DnsConnectionGetProxyList@8
DnsConnectionSetPolicyEntries@8
DnsConnectionSetPolicyEntriesPrivate@12
DnsConnectionSetProxyInfo@12
DnsConnectionUpdateIfIndexTable@4
DnsCopyStringEx@20
DnsCreateReverseNameStringForIpAddress@4
DnsCreateStandardDnsNameCopy@12
DnsCreateStringCopy@8
DnsDeRegisterLocal@8
DnsDhcpRegisterAddrs@4
DnsDhcpRegisterHostAddrs@40
DnsDhcpRegisterInit
DnsDhcpRegisterTerm
DnsDhcpRemoveRegistrations
DnsDhcpSrvRegisterHostAddr@4
DnsDhcpSrvRegisterHostAddrEx@4
DnsDhcpSrvRegisterHostName@48
DnsDhcpSrvRegisterHostNameEx@60
DnsDhcpSrvRegisterInit@8
DnsDhcpSrvRegisterInitEx@12
DnsDhcpSrvRegisterInitialize@4
DnsDhcpSrvRegisterTerm
DnsDisableIdnEncoding@8
DnsDowncaseDnsNameLabel@16
DnsExtractRecordsFromMessage_UTF8@12
DnsExtractRecordsFromMessage_W@12
DnsFindAuthoritativeZone@16
DnsFlushResolverCache
DnsFlushResolverCacheEntry_A@4
DnsFlushResolverCacheEntry_UTF8@4
DnsFlushResolverCacheEntry_W@4
DnsFree@8
DnsFreeAdaptersInfo@8
DnsFreeConfigStructure@8
DnsFreeNrptRule@4
DnsFreeNrptRuleNamesList@8
DnsFreePolicyConfig@4
DnsFreeProxyName@4
DnsGetAdaptersInfo@24
DnsGetApplicationIdentifier@12
DnsGetBufferLengthForStringCopy@16
DnsGetCacheDataTable@4
DnsGetCacheDataTableEx@12
DnsGetDnsServerList@4
DnsGetInterfaceSettings@20
DnsGetLastFailedUpdateInfo@4
DnsGetNrptRuleNamesList@8
DnsGetPolicyTableInfo@16
DnsGetPolicyTableInfoPrivate@16
DnsGetPrimaryDomainName_A
DnsGetProxyInfoPrivate@16
DnsGetProxyInformation@20
DnsGetQueryRetryTimeouts@24
DnsGetSettings@4
DnsGlobals DATA
DnsIpv6AddressToString@8
DnsIpv6StringToAddress@12
DnsIsStringCountValidForTextType@8
DnsLogEvent@16
DnsModifyRecordsInSet_A@24
DnsModifyRecordsInSet_UTF8@24
DnsModifyRecordsInSet_W@24
DnsNameCompareEx_A@12
DnsNameCompareEx_UTF8@12
DnsNameCompareEx_W@12
DnsNameCompare_A@8
DnsNameCompare_UTF8@8
DnsNameCompare_W@8
DnsNameCopy@24
DnsNameCopyAllocate@16
DnsNetworkInfo_CreateFromFAZ@20
DnsNetworkInformation_CreateFromFAZ@16
DnsNotifyResolver@8
DnsNotifyResolverClusterIp@8
DnsNotifyResolverEx@16
DnsQueryConfig@24
DnsQueryConfigAllocEx@12
DnsQueryConfigDword@8
DnsQueryEx@12
DnsQueryExA@4
DnsQueryExUTF8@4
DnsQueryExW@4
DnsQuery_A@24
DnsQuery_UTF8@24
DnsQuery_W@24
DnsRecordBuild_UTF8@28
DnsRecordBuild_W@28
DnsRecordCompare@8
DnsRecordCopyEx@12
DnsRecordListFree@8
DnsRecordListUnmapV4MappedAAAAInPlace@4
DnsRecordSetCompare@16
DnsRecordSetCopyEx@12
DnsRecordSetDetach@4
DnsRecordStringForType@4
DnsRecordStringForWritableType@4
DnsRecordTypeForName@8
DnsRegisterLocal@16
DnsReleaseContextHandle@4
DnsRemoveNrptRule@4
DnsRemoveRegistrations
DnsReplaceRecordSetA@20
DnsReplaceRecordSetUTF8@20
DnsReplaceRecordSetW@20
DnsResetQueryRetryTimeouts@16
DnsResolverOp@12
DnsResolverQueryHvsi@32
DnsScreenLocalAddrsForRegistration@12
DnsServiceBrowse@8
DnsServiceBrowseCancel@4
DnsServiceConstructInstance@40
DnsServiceCopyInstance@4
DnsServiceDeRegister@8
DnsServiceFreeInstance@4
DnsServiceRegister@8
DnsServiceRegisterCancel@4
DnsServiceResolve@8
DnsServiceResolveCancel@4
DnsSetConfigDword@12
DnsSetConfigValue@20
DnsSetInterfaceSettings@20
DnsSetNrptRule@12
DnsSetNrptRules@16
DnsSetQueryRetryTimeouts@24
DnsSetSettings@4
DnsStartMulticastQuery@8
DnsStopMulticastQuery@4
DnsStringCopyAllocateEx@16
DnsTraceServerConfig@12
DnsUpdate@20
DnsUpdateMachinePresence
DnsUpdateTest_A@16
DnsUpdateTest_UTF8@16
DnsUpdateTest_W@16
DnsValidateNameOrIp_TempW@8
DnsValidateName_A@8
DnsValidateName_UTF8@8
DnsValidateName_W@8
DnsValidateServerArray_A@12
DnsValidateServerArray_W@12
DnsValidateServerStatus@12
DnsValidateServer_A@12
DnsValidateServer_W@12
DnsValidateUtf8Byte@8
DnsWriteQuestionToBuffer_UTF8@24
DnsWriteQuestionToBuffer_W@24
DnsWriteReverseNameStringForIpAddress@8
Dns_AddRecordsToMessage@12
Dns_AllocateMsgBuf@4
Dns_BuildPacket@28
Dns_CacheServiceCleanup
Dns_CacheServiceInit
Dns_CacheServiceStopIssued
Dns_CleanupWinsock@0
Dns_CloseConnection@4
Dns_CloseSocket@4
Dns_CreateMulticastSocket@20
Dns_CreateSocket@12
Dns_CreateSocketEx@20
Dns_ExtractRecordsFromMessage@12
Dns_FindAuthoritativeZoneLib@16
Dns_FreeMsgBuf@4
Dns_GetRandomXid@4
Dns_InitializeMsgBuf@4
Dns_InitializeMsgRemoteSockaddr@8
Dns_InitializeWinsock
Dns_OpenTcpConnectionAndSend@12
Dns_ParseMessage@20
Dns_ParsePacketRecord@12
Dns_PingAdapterServers@4
Dns_ReadRecordStructureFromPacket@12
Dns_RecvTcp@4
Dns_ResetNetworkInfo@4
Dns_SendAndRecvUdp@20
Dns_SendEx@12
Dns_SetRecordDatalength@8
Dns_SetRecordsSection@8
Dns_SetRecordsTtl@8
Dns_SkipToRecord@12
Dns_UpdateLib@20
Dns_UpdateLibEx@28
Dns_WriteQuestionToMessage@16
Dns_WriteRecordStructureToPacketEx@20
ExtraInfo_Init@8
Faz_AreServerListsInSameNameSpace@12
FlushDnsPolicyUnreachableStatus
GetCurrentTimeInSeconds
HostsFile_Close@4
HostsFile_Open@4
HostsFile_ReadLine@4
IpHelp_IsAddrOnLink@4
Local_GetRecordsForLocalName@8
Local_GetRecordsForLocalNameEx@20
NetInfo_Build@8
NetInfo_Clean@8
NetInfo_Copy@4
NetInfo_CopyNetworkIndex@8
NetInfo_CreatePerNetworkNetinfo@8
NetInfo_Free@4
NetInfo_GetAdapterByAddress@12
NetInfo_GetAdapterByInterfaceIndex@12
NetInfo_GetAdapterByName@8
NetInfo_IsAddrConfig@8
NetInfo_IsForUpdate@4
NetInfo_IsTcpipConfigChange@4
NetInfo_ResetServerPriorities@8
NetInfo_UpdateDnsInterfaceConfigChange@4
NetInfo_UpdateNetworkProperties@28
NetInfo_UpdateServerReachability@12
QueryDirectEx@40
Query_Cancel@12
Query_Main@4
Reg_FreeUpdateInfo@8
Reg_GetValueEx@28
Reg_ReadGlobalsEx@8
Reg_ReadUpdateInfo@8
Security_ContextListTimeout@4
Send_AndRecvUdpWithParam@4
Send_MessagePrivate@12
Send_MessagePrivateEx@16
Send_OpenTcpConnectionAndSend@12
Socket_CacheCleanup@0
Socket_CacheInit@4
Socket_CleanupWinsock@0
Socket_ClearMessageSockets@4
Socket_CloseEx@8
Socket_CloseMessageSockets@4
Socket_Create@20
Socket_CreateMulticast@20
Socket_InitWinsock@4
Socket_JoinMulticast@20
Socket_RecvFrom@40
Socket_SetMulticastInterface@16
Socket_SetMulticastLoopBack@12
Socket_SetTtl@20
Socket_TcpListen@4
Trace_Reset@0
Update_ReplaceAddressRecordsW@20
Util_IsIp6Running@0
Util_IsRunningOnXboxOne@0
WriteDnsNrptRulesToRegistry@16
