;
; Definition file of TetheringStation.dll
; Automatic generated by gendef
; written by Kai Tietz 2008-2014
;
LIBRARY "TetheringStation.dll"
EXPORTS
TetheringStationConnect
TetheringStationDeinitialize
TetheringStationDisconnect
TetheringStationEnumerate
TetheringStationFreeMemory
TetheringStationInitialize
TetheringStationRegisterForNotification
TetheringStationUnregisterForNotification
