; 
; Exports of file mofd.dll
;
; Autogenerated by gen_exportdef
; Written by Kai Tietz, 2007
;
LIBRARY mofd.dll
EXPORTS
CompileFileViaDLL
CreateBMOFViaDLL
DllCanUnloadNow
DllGetClassObject
DllRegisterServer
DllUnregisterServer
