;
; Definition file of oledlg.dll
; Automatic generated by gendef
; written by Kai Tietz 2008
;
LIBRARY "oledlg.dll"
EXPORTS
OleUIAddVerbMenuA@36
OleUICanConvertOrActivateAs@12
OleUIInsertObjectA@4
OleUIPasteSpecialA@4
OleUIEditLinksA@4
OleUIChangeIconA@4
OleUIConvertA@4
OleUIBusyA@4
OleUIUpdateLinksA@16
OleUIPromptUserA
OleUIObjectPropertiesA@4
OleUIChangeSourceA@4
OleUIAddVerbMenuW@36
OleUIBusyW@4
OleUIChangeIconW@4
OleUIChangeSourceW@4
OleUIConvertW@4
OleUIEditLinksW@4
OleUIInsertObjectW@4
OleUIObjectPropertiesW@4
OleUIPasteSpecialW@4
OleUIPromptUserW
OleUIUpdateLinksW@16
