; 
; Exports of file odbcconf.DLL
;
; Autogenerated by gen_exportdef
; Written by Kai Tietz, 2007
;
LIBRARY odbcconf.DLL
EXPORTS
SetSilent
SetActionLogFile
SetActionLogModeSz
SetActionLogMode
ExecuteAction
SetActionEnum
SetActionName
ExpandPath
GetPathFromID
ApplyW2KXPak
ApplyMillenCat
UnregW2KXPak
AppRegEnum
CloseAppRegEnum
OpenAppRegEnum
QueryApplication
RefreshAppRegEnum
RegisterApplication
RunDLL32_RegisterApplication
RunDLL32_UnregisterApplication
UnregisterApplication
DllCanUnloadNow
DllGetClassObject
DllRegisterServer
SLListCreate
SLListDestroy
SLListAddNode
SLListRemoveNode
SLListGetHead
SLListGetNext
SLListGetData
SLListSetData
DllUnregisterServer
DLListCreate
DLListDestroy
DLListAddNode
DLListRemoveNode
DLListGetHead
DLListGetNext
DLListGetPrev
DLListGetData
DLListSetData
OpenProcessInfo
CloseProcessInfo
IsFileLocked
EnableNTPrivilege
DisableNTPrivilege
EnableDebugPrivileges
DisableDebugPrivileges
LookupAppFriendlyName
IsModuleLoadedByProcess
IsModuleLoadedByProcessEx
ShutdownProcess
OpenProcessDisplayNames
CloseProcessDisplayNames
GetProcessListFromFileList
OpenProcessList
CloseProcessList
EnumerateProcesses
AddInfToList
CreateInfList
PopulateInfFileList
DestroyInfList
OpenDriveSpaceList
CloseDriveSpaceList
CalculateRequiredDriveSpace
OpenDriveSpaceListFromInfList
RunDLL32_FilterRunOnceExRegistration
AddFilterRunOnceExRegistrationToRegistry
RemoveFilterRunOnceExRegistrationFromRegistry
FilterRunOnceExRegistration
BackupRegKey
RestoreRegKey
RegVersion
RegVersionEx
LoadRegVersion
LoadRegVersionEx
GetProgramFilesDirectory
GetProgramFilesCommonFilesDirectory
OpenPendingRenameList
ClosePendingRenameList
ParseVersionDataFromString
ParseVersionDataFromStringEx
CompareVersionData
CompareVersionDataEx
IsExceptionInf
IsExceptionInfEx
CreateRegKeyBackupList
DestroyRegKeyBackupList
RegKeyBackup
RegKeyRestore
SaveRegKeyBackupToFile
LoadRegKeyBackupFromFile
BackupCatalog
RestoreCatalog
DoesFileExist
