;
; Definition file of bfe.dll
; Automatic generated by gendef
; written by Kai Tietz 2008-2014
;
LIBRARY "bfe.dll"
EXPORTS
BfeGetDirectDispatchTable
BfeOnServiceStartTypeChange
BfeServiceMain
SvchostPushServiceGlobals
