;
; Definition file of eappprxy.dll
; Automatic generated by gendef
; written by Kai Tietz 2008
;
LIBRARY "eappprxy.dll"
EXPORTS
EapHostPeerBeginSession
EapHostPeerClearConnection
EapHostPeerEndSession
EapHostPeerFreeEapError
EapHostPeerFreeRuntimeMemory
EapHostPeerGetAuthStatus
EapHostPeerGetIdentity
EapHostPeerGetResponseAttributes
EapHostPeerGetResult
EapHostPeerGetSendPacket
EapHostPeerGetUIContext
EapHostPeerInitialize
EapHostPeerProcessReceivedPacket
EapHostPeerSetResponseAttributes
EapHostPeerSetUIContext
EapHostPeerUninitialize
