;
; Definition file of bitsigd.dll
; Automatic generated by gendef
; written by Kai Tietz 2008-2014
;
LIBRARY "bitsigd.dll"
EXPORTS
?s_EmptyString@?$GenericStringHandle@G@@0UStringData@1@A DATA
InitializeEx
UninitializeEx
