;
; Definition file of sppwinob.dll
; Automatic generated by gendef
; written by Kai Tietz 2008-2014
;
LIBRARY "sppwinob.dll"
EXPORTS
SppPluginCanUnloadNow
SppPluginCreateInstance
SppPluginInitialize
SppPluginShutdown
SppPluginVersion
