; 
; Exports of file SQLXMLX.dll
;
; Autogenerated by gen_exportdef
; Written by Kai Tietz, 2007
;
LIBRARY SQLXMLX.dll
EXPORTS
ExecuteToStream
DllCanUnloadNow
DllGetClassObject
DllRegisterServer
DllUnregisterServer
