; 
; Exports of file SCARDDLG.dll
;
; Autogenerated by gen_exportdef
; Written by Kai Tietz, 2007
;
LIBRARY SCARDDLG.dll
EXPORTS
GetOpenCardNameA
GetOpenCardNameW
SCardDlgExtendedError
SCardUIDlgSelectCardA
SCardUIDlgSelectCardW
SCardUIDlgGetPINA
SCardUIDlgChangePINA
