;
; Definition file of wuaueng.dll
; Automatic generated by gendef
; written by Kai Tietz 2008-2014
;
LIBRARY "wuaueng.dll"
EXPORTS
GetAUOptionsEx
GeneralizeForImaging
ord_3 @3
WUCheckForUpdatesAtShutdown
WUAutoUpdateAtShutdown
GetEngineStatusInfo
RegisterServiceVersion
ServiceHandler
ServiceMain
WUServiceMain
