; 
; Exports of file mstlsapi.dll
;
; Autogenerated by gen_exportdef
; Written by Kai Tietz, 2007
;
LIBRARY mstlsapi.dll
EXPORTS
TLSGetVersion
MIDL_user_allocate
MIDL_user_free
EnumerateTlsServer
TLSSendServerCertificate
TLSGetServerName
TLSGetServerScope
TLSIssuePlatformChallenge
TLSIssueNewLicense
TLSUpgradeLicense
TLSAllocateConcurrentLicense
TLSGetLastError
TLSKeyPackEnumBegin
TLSKeyPackEnumNext
TLSKeyPackEnumEnd
TLSLicenseEnumBegin
TLSLicenseEnumNext
TLSLicenseEnumEnd
TLSGetAvailableLicenses
TLSConnectToLsServer
TLSConnectToAnyLsServer
TLSDisconnectFromServer
FindEnterpriseServer
GetAllEnterpriseServers
TLSInit
TLSGetTSCertificate
LsCsp_GetServerData
LsCsp_DecryptEnvelopedData
LsCsp_EncryptHwid
LsCsp_StoreSecret
LsCsp_RetrieveSecret
TLSStartDiscovery
TLSStopDiscovery
TLSShutdown
TLSFreeTSCertificate
TLSIssueNewLicenseEx
TLSUpgradeLicenseEx
TLSCheckLicenseMark
TLSIssueNewLicenseExEx
TLSGetServerNameEx
TLSLicenseEnumNextEx
TLSGetServerNameFixed
TLSGetServerScopeFixed
TLSGetLastErrorFixed
GetLicenseServersFromReg
TLSConnectToAnyLsServerNoCertInstall
RequestToTlsRequest
TLSRequestTermServCert
TLSRetrieveTermServCert
TLSInstallCertificate
TLSGetServerCertificate
TLSRegisterLicenseKeyPack
TLSGetLSPKCS10CertRequest
TLSKeyPackAdd
TLSKeyPackSetStatus
TLSReturnLicense
TLSAnnounceServer
TLSLookupServer
TLSAnnounceLicensePack
TLSReturnLicensedProduct
TLSTelephoneRegisterLKP
TLSChallengeServer
TLSResponseServerChallenge
TLSGetTlsPrivateData
TLSTriggerReGenKey
TLSGetServerPID
TLSGetServerSPK
TLSDepositeServerSPK
TLSAllocateInternetLicenseEx
TLSReturnInternetLicenseEx
TLSIsBetaNTServer
TLSIsLicenseEnforceEnable
TLSInDomain
TLSMarkLicense
TLSGetSupportFlags
TLSLookupServerFixed
