;
; Definition file of LSASRV.dll
; Automatic generated by gendef
; written by Kai Tietz 2008-2014
;
LIBRARY "LSASRV.dll"
EXPORTS
InitializeLsaExtension
QueryLsaInterface
LsaDbLookupSidChainRequest
LsaIAddCredentialKeys
LsaIAddNamesToLogonSession
LsaIAdjustTokenObjectIntegrity
LsaIAdtAuditingEnabledByCategory
LsaIAdtAuditingEnabledBySubCategory
LsaIAllocateHeap
LsaIAllocateHeapZero
LsaIAuditAccountLogon
LsaIAuditAccountLogonEx
LsaIAuditInitializeParametersAndWriteEvent
LsaIAuditKdcEvent
LsaIAuditKerberosLogon
LsaIAuditLogonEx
LsaIAuditLogonUsingExplicitCreds
LsaIAuditNotifyPackageLoad
LsaIAuditPasswordAccessEvent
LsaIAuditReplay
LsaIAuditSamEvent
LsaICallPackage
LsaICallPackageEx
LsaICallPackagePassthrough
LsaICancelNotification
LsaIChangeSecretCipherKey
LsaICheckProtectedUserByTokenInfo
LsaIClearOldSyskey
LsaICryptProtectData
LsaICryptProtectDataEx
LsaICryptUnprotectData
LsaICryptUnprotectDataEx
LsaIDereferenceCredHandle
LsaIDeriveAndEncodeCredentialKeys
LsaIDsNotifiedObjectChange
LsaIEfsAcceptSmartcardCredentials
LsaIEqualLogonProcessName
LsaIEqualSupplementalTokenInfo
LsaIEventWritePackageNoCredential
LsaIEventWritePackageNotCacheLogonUser
LsaIFilterNamespace
LsaIFilterSids
LsaIForestTrustFindMatch
LsaIFreeForestTrustInfo
LsaIFreeHeap
LsaIFreeReturnBuffer
LsaIFreeSupplementalTokenInfo
LsaIFree_LSAI_PRIVATE_DATA
LsaIFree_LSAI_SECRET_ENUM_BUFFER
LsaIFree_LSAPR_ACCOUNT_ENUM_BUFFER
LsaIFree_LSAPR_CR_CIPHER_VALUE
LsaIFree_LSAPR_POLICY_DOMAIN_INFORMATION
LsaIFree_LSAPR_POLICY_INFORMATION
LsaIFree_LSAPR_PRIVILEGE_ENUM_BUFFER
LsaIFree_LSAPR_PRIVILEGE_SET
LsaIFree_LSAPR_REFERENCED_DOMAIN_LIST
LsaIFree_LSAPR_SR_SECURITY_DESCRIPTOR
LsaIFree_LSAPR_TRANSLATED_NAMES
LsaIFree_LSAPR_TRANSLATED_SIDS
LsaIFree_LSAPR_TRUSTED_DOMAIN_INFO
LsaIFree_LSAPR_TRUSTED_ENUM_BUFFER
LsaIFree_LSAPR_TRUSTED_ENUM_BUFFER_EX
LsaIFree_LSAPR_TRUST_INFORMATION
LsaIFree_LSAPR_UNICODE_STRING
LsaIFree_LSAPR_UNICODE_STRING_BUFFER
LsaIFree_LSAP_SITENAME_INFO
LsaIFree_LSAP_SITE_INFO
LsaIFree_LSAP_SUBNET_INFO
LsaIFree_LSAP_UPN_SUFFIXES
LsaIFree_LSA_FOREST_TRUST_COLLISION_INFORMATION
LsaIFree_LSA_FOREST_TRUST_INFORMATION
LsaIGetCallInfo
LsaIGetForestTrustInformation
LsaIGetLogonGuid
LsaIGetNameFromLuid
LsaIGetNbAndDnsDomainNames
LsaIGetNego2Package
LsaIGetSiteName
LsaIGetSupplementalTokenInfo
LsaIHealthCheck
LsaIImpersonateClient
LsaIInitializeNetlogonFuncPtrs
LsaIIsDomainWithinForest
LsaIIsDsPaused
LsaIIsLastInteractiveLogonInfoEnabled
LsaIIsLocalHost
LsaIIsSuppressChannelBindingInfo
LsaIIsTrustedDomainsEnabled
LsaIKerberosRegisterTrustNotification
LsaILookupWellKnownName
LsaIModifyPerformanceCounter
LsaINoConnectedUserPolicy
LsaINoMoreWin2KDomain
LsaINotifyChangeNotification
LsaINotifyGCStatusChange
LsaINotifyNetlogonParametersChangeW
LsaINotifyNewPassword
LsaINotifyPasswordChanged
LsaIOpenPolicyTrusted
LsaIQueryForestTrustInfo
LsaIQueryInformationPolicyTrusted
LsaIQueryPackageAttrInLogonSession
LsaIQuerySiteInfo
LsaIQuerySubnetInfo
LsaIQueryUpnSuffixes
LsaIReferenceCredHandle
LsaIRegisterLogonSessionCallback
LsaIRegisterNotification
LsaIRegisterPolicyChangeNotificationCallback
LsaIReplicateClientObject
LsaIRetrieveCurrentUserSid
LsaISafeMode
LsaISamIndicatedDsStarted
LsaISetClientDnsHostName
LsaISetLogonGuidInLogonSession
LsaISetLogonInfo
LsaISetNewSyskey
LsaISetPackageAttrInLogonSession
LsaISetSupplementalTokenInfo
LsaISetTokenDacl
LsaISetUserFlags
LsaISetupWasRun
LsaITransformAuthorizationData
LsaIUnregisterAllPolicyChangeNotificationCallback
LsaIUnregisterLogonSessionCallback
LsaIUnregisterPolicyChangeNotificationCallback
LsaIUpdateForestTrustInformation
LsaIUpdateKerbMaxTokenSize
LsaIUpdateLogonSession
LsaIValidateTargetInfo
LsaIVerifyCachability
LsaIWriteAuditEvent
LsaIWriteKdcAuthenticationEvent
LsapAdtAuditingEnabledByLogonId
LsapAdtAuditingEnabledBySubCategory
LsapAdtAuditingEnabledHint
LsapAdtInitParametersArray
LsapAdtWriteLog
LsapAllocateLsaHeap
LsapAllocatePrivateHeap
LsapAuOpenSam
LsapAuditFailed
LsapBuildPrivilegeAuditString
LsapCheckBootMode
LsapCloseHandle
LsapCompareDomainNames
LsapCrServerGetSessionKey
LsapCrServerGetSessionKeySafe
LsapDbAcquireLockEx
LsapDbApplyTransaction
LsapDbBuildObjectCaches
LsapDbCloseHandle
LsapDbCloseObject
LsapDbCopyUnicodeAttribute
LsapDbCopyUnicodeAttributeNoAlloc
LsapDbCreateObject
LsapDbDeleteAttributesObject
LsapDbDeleteObject
LsapDbDereferenceHandle
LsapDbDereferenceObject
LsapDbEnumerateSids
LsapDbEnumerateTrustedDomainsEx
LsapDbExpAcquireReadLockTrustedDomainList
LsapDbExpAcquireWriteLockTrustedDomainList
LsapDbExpConvertReadLockTrustedDomainListToExclusive
LsapDbExpConvertWriteLockTrustedDomainListToShared
LsapDbExpIsCacheBuilding
LsapDbExpIsCacheValid
LsapDbExpIsLockedTrustedDomainList
LsapDbExpMakeCacheBuilding
LsapDbExpMakeCacheInvalid
LsapDbExpMakeCacheValid
LsapDbExpReleaseLockTrustedDomainList
LsapDbFreeAttributes
LsapDbFreeTrustedDomainsEx
LsapDbGetDbObjectTypeName
LsapDbGetDbPolicyHandle
LsapDbGetSecretType
LsapDbInitializeAttribute
LsapDbIsStatusConnectionFailure
LsapDbLookupAddListReferencedDomains
LsapDbLookupCreateListReferencedDomains
LsapDbLookupGetDomainInfo
LsapDbLookupListReferencedDomains
LsapDbLookupMergeDisjointReferencedDomains
LsapDbLookupNameChainRequest
LsapDbLookupNamesInPrimaryDomain
LsapDbLookupSidsInPrimaryDomain
LsapDbMakeGuidAttribute
LsapDbMakeSidAttribute
LsapDbMakeUnicodeAttribute
LsapDbOpenObject
LsapDbQueryInformationPolicy
LsapDbReadAttribute
LsapDbReadAttributesObject
LsapDbReferenceObject
LsapDbReleaseLockEx
LsapDbSecretIsMachineAcc
LsapDbSidToLogicalNameObject
LsapDbSlowEnumerateTrustedDomains
LsapDbUpdateCountCompUnmappedNames
LsapDbVerifyHandle
LsapDbVerifyInfoQueryTrustedDomain
LsapDbVerifyInfoSetTrustedDomain
LsapDbWriteAttributesObject
LsapDomainRenameHandlerForLogonSessions
LsapDsInitializeDsStateInfo
LsapDsUnitializeDsStateInfo
LsapDssetupInitializeGetPrimaryDomainInformationOpState
LsapDuplicateSid
LsapDuplicateString
LsapFreeLsaHeap
LsapFreePrivateHeap
LsapFreeString
LsapGetAccountDomainHandle
LsapGetCapeNamesForCap
LsapGetGlobalRestrictAnonymous
LsapGetHourlyLogLevel
LsapGetLogonSessionAccountInfoEx
LsapGetLookupRestrictIsolatedNameLevel
LsapGetPolicyHandle
LsapGetWellKnownSid
LsapInitLsa
LsapInitializeLsaDb
LsapIsBuiltinDomain
LsapIsSamOpened
LsapOpenSam
LsapQueryClientInfo
LsapRemoveTrailingDot
LsapRpcCopySid
LsapRpcCopyUnicodeString
LsapRtlValidateControllerTrustedDomain
LsapRtlValidateControllerTrustedDomainByHandle
LsapSetErrorInfo
LsapSidListSize
LsapTraceEvent
LsapTraceEventWithData
LsapTruncateUnicodeString
LsarClose
LsarCreateSecret
LsarDeleteObject
LsarEnumerateTrustedDomainsEx
LsarLookupSids
LsarOpenPolicy
LsarOpenSecret
LsarQueryDomainInformationPolicy
LsarQueryInformationPolicy
LsarQuerySecret
LsarQueryTrustedDomainInfoByName
LsarRetrievePrivateData
LsarSetInformationPolicy
LsarSetSecret
LsarSetTrustedDomainInfoByName
LsarStorePrivateData
ServiceInit
_fgs__LSAPR_TRUSTED_ENUM_BUFFER
_fgs__LSAPR_TRUSTED_ENUM_BUFFER_EX
_fgs__LSAPR_TRUST_INFORMATION
_fgu__LSAPR_TRUSTED_DOMAIN_INFO
