;
; Definition file of twinapi.appcore.dll
; Automatic generated by gendef
; written by Kai Tietz 2008-2014
;
LIBRARY "twinapi.appcore.dll"
EXPORTS
ord_1 @1
ord_2 @2
ord_3 @3
ord_4 @4
ord_5 @5
ord_6 @6
ord_7 @7
ord_8 @8
ord_9 @9
ord_10 @10
ord_11 @11
ord_12 @12
BiChangeApplicationStateForPackageName
BiChangeApplicationStateForPsmKey
BiChangeSessionState
BiGetActiveBackgroundTasksEvent
BiIsApplicationTerminateSensitive
BiNotifyNewSession
BiPtActivateDeferredWorkItem
BiPtActivateInBackground
BiPtActivateWorkItem
BiPtAssociateActivationProxy
BiPtAssociateApplicationExtensionClass
BiPtCancelWorkItem
BiPtCreateEventForPackageName
BiPtDeleteEvent
BiPtDisassociateWorkItem
BiPtEnumerateBrokeredEvents
BiPtEnumerateWorkItemsForPackageName
BiPtFreeMemory
BiPtQueryBrokeredEvent
BiPtQuerySystemStateBroadcastChannels
BiPtQueryWorkItem
BiPtSignalEvent
BiPtSignalMultipleEvents
BiResetActiveSessionForPackage
BiSetActiveSessionForPackage
BiUpdateLockScreenApplications
PsmApplyTaskCompletion
PsmBlockAppStateChangeCompletion
PsmDisconnect
PsmIsProcessInApplication
PsmQueryApplicationInformation
PsmQueryApplicationInterferenceCount
PsmQueryApplicationList
PsmQueryApplicationProperties
PsmQueryApplicationResourceUsage
PsmQueryCurrentAppState
PsmQueryMaxMemoryUsage
PsmQueryProcessList
PsmQueryTaskCompletionInformation
PsmRegisterAppStateChangeNotification
PsmRegisterApplicationNotification
PsmRegisterDynamicProcess
PsmRegisterKeyNotification
PsmSetApplicationPriority
PsmSetApplicationProperties
PsmSetApplicationState
PsmShutdownApplication
PsmUnblockAppStateChangeCompletion
PsmUnregisterAppStateChangeNotification
PsmWaitForAppResume
RegisterAppStateChangeNotification
UnregisterAppStateChangeNotification
