; 
; Exports of file COMPSTUI.dll
;
; Autogenerated by gen_exportdef
; Written by Kai Tietz, 2007
;
LIBRARY COMPSTUI.dll
EXPORTS
CommonPropertySheetUIA
CommonPropertySheetUIW
GetCPSUIUserData
SetCPSUIUserData
