; 
; Exports of file SYNCENG.dll
;
; Autogenerated by gen_exportdef
; Written by Kai Tietz, 2007
;
LIBRARY SYNCENG.dll
EXPORTS
AddAllTwinsToTwinList
AddFolderTwin
AddObjectTwin
AddTwinToTwinList
AnyTwins
BeginReconciliation
ClearBriefcaseCache
CloseBriefcase
CompareFileStamps
CountSourceFolderTwins
CreateFolderTwinList
CreateRecList
CreateTwinList
DeleteBriefcase
DeleteTwin
DestroyFolderTwinList
DestroyRecList
DestroyTwinList
EndReconciliation
FindBriefcaseClose
FindFirstBriefcase
FindNextBriefcase
GetFileStamp
GetFolderTwinStatus
GetObjectTwinHandle
GetOpenBriefcaseInfo
GetVolumeDescription
IsFolderTwin
IsOrphanObjectTwin
IsPathOnVolume
OpenBriefcase
ReconcileItem
ReleaseTwinHandle
RemoveAllTwinsFromTwinList
RemoveTwinFromTwinList
SaveBriefcase
