LIBRARY api-ms-win-devices-config-l1-1-2

EXPORTS

CM_Delete_Device_Interface_KeyA@8
CM_Get_Class_Registry_PropertyA@28
CM_Get_Device_ID_List_SizeA@12
CM_Get_Device_ID_ListA@16
CM_Get_Device_IDA@16
CM_Get_Device_Interface_List_SizeA@16
CM_Get_Device_Interface_ListA@20
CM_Get_DevNode_Registry_PropertyA@24
CM_Locate_DevNodeA@12
CM_Open_Class_KeyA@24
CM_Open_Device_Interface_KeyA@20
CM_Query_And_Remove_SubTreeA@20
CM_Set_Class_Registry_PropertyA@24
CM_Set_DevNode_Registry_PropertyA@20
