;
; Definition file of DEVRTL.dll
; Automatic generated by gendef
; written by Kai Tietz 2008
;
LIBRARY "DEVRTL.dll"
EXPORTS
DevRtlCloseTextLogSection@12
DevRtlCreateTextLogSectionA@16
DevRtlCreateTextLogSectionW@16
DevRtlGetThreadLogToken@0
DevRtlSetThreadLogToken@8
DevRtlWriteTextLog@0
DevRtlWriteTextLogError@0
NdxTableAddObject@16
NdxTableAddObjectToList@12
NdxTableClose@8
NdxTableFirstObject@12
NdxTableFirstObjectInList@12
NdxTableGetObjectName@16
NdxTableGetObjectType@8
NdxTableGetObjectTypeCount@8
NdxTableGetObjectTypeName@20
NdxTableGetPropertyTypeClass@16
NdxTableGetPropertyTypeCount@12
NdxTableGetPropertyTypeName@24
NdxTableGetPropertyValue@20
NdxTableNextObject@4
NdxTableObjectFromName@16
NdxTableObjectFromPointer@12
NdxTableOpen@16
NdxTableRemoveObject@4
NdxTableRemoveObjectFromList@12
NdxTableSetObjectPointer@12
NdxTableSetPropertyValue@16
NdxTableSetTypeDefinition@16
