;
; Definition file of dxgi.dll
; Automatic generated by gendef
; written by Kai Tietz 2008
;
LIBRARY "dxgi.dll"
EXPORTS
ApplyCompatResolutionQuirking@8
CompatString@16
CompatValue@8
D3DKMTCloseAdapter@4
D3DKMTDestroyAllocation@4
D3DKMTDestroyContext@4
D3DKMTDestroyDevice@4
D3DKMTDestroySynchronizationObject@4
D3DKMTQueryAdapterInfo@4
D3DKMTSetDisplayPrivateDriverFormat@4
D3DKMTSignalSynchronizationObject@4
D3DKMTUnlock@4
D3DKMTWaitForSynchronizationObject@0
DXGIDumpJournal@4
PIXBeginCapture@8
PIXEndCapture@4
PIXGetCaptureState@0
DXGIRevertToSxS@0
OpenAdapter10@4
OpenAdapter10_2@4
SetAppCompatStringPointer@8
UpdateHMDEmulationStatus@4
CreateDXGIFactory1@8
CreateDXGIFactory2@12
CreateDXGIFactory@8
D3DKMTCreateAllocation@4
D3DKMTCreateContext@4
D3DKMTCreateDevice@4
D3DKMTCreateSynchronizationObject@4
D3DKMTEscape@4
D3DKMTGetContextSchedulingPriority@4
D3DKMTGetDeviceState@4
D3DKMTGetDisplayModeList@4
D3DKMTGetMultisampleMethodList@4
D3DKMTGetRuntimeData@4
D3DKMTGetSharedPrimaryHandle@4
D3DKMTLock@4
D3DKMTOpenAdapterFromHdc@4
D3DKMTOpenResource@4
D3DKMTPresent@4
D3DKMTQueryAllocationResidency@4
D3DKMTQueryResourceInfo@4
D3DKMTRender@4
D3DKMTSetAllocationPriority@4
D3DKMTSetContextSchedulingPriority@4
D3DKMTSetDisplayMode@4
D3DKMTSetGammaRamp@4
D3DKMTSetVidPnSourceOwner@4
D3DKMTWaitForSynchronizationObject@4
D3DKMTWaitForVerticalBlankEvent@4
DXGID3D10CreateDevice@24
DXGID3D10CreateLayeredDevice@20
DXGID3D10ETWRundown@0
DXGID3D10GetLayeredDeviceSize@8
DXGID3D10RegisterLayers@8
DXGIDeclareAdapterRemovalSupport@0
DXGIGetDebugInterface1@12
DXGIReportAdapterConfiguration@4
