;
; Definition file of RstrtMgr.DLL
; Automatic generated by gendef
; written by Kai Tietz 2008
;
LIBRARY "RstrtMgr.DLL"
EXPORTS
RmAddFilter@20
RmCancelCurrentTask@4
RmEndSession@4
RmGetFilterList@16
RmGetList@20
RmJoinSession@8
RmRegisterResources@28
RmRemoveFilter@16
RmReserveHeap@4
RmRestart@12
RmShutdown@12
RmStartSession@12
