/**
 * This file has no copyright assigned and is placed in the Public Domain.
 * This file is part of the mingw-w64 runtime package.
 * No warranty is given; refer to the file DISCLAIMER.PD within this package.
 */
#define VER_MAJOR_PRODUCTVER 6
#define VER_MINOR_PRODUCTVER 0

#define VER_PRODUCTVERSION VER_MAJOR_PRODUCTVER,VER_MINOR_PRODUCTVER,VER_PRODUCTBUILD,VER_PRODUCTBUILD_QFE
#define VER_PRODUCTVERSION_W (0x0600)
#define VER_PRODUCTVERSION_DW (0x06000000 | VER_PRODUCTBUILD)

#define VER_PRODUCTBETA_STR ""
#define VER_PRODUCTVERSION_STRING "6.00"
