;
; Definition file of dsrole.dll
; Automatic generated by gendef
; written by Kai Tietz 2008
;
LIBRARY "dsrole.dll"
EXPORTS
DsRoleAbortDownlevelServerUpgrade
DsRoleCancel
DsRoleDcAsDc
DsRoleDcAsReplica
DsRoleDemoteDc
DsRoleDnsNameToFlatName
DsRoleFreeMemory
DsRoleGetDatabaseFacts
DsRoleGetDcOperationProgress
DsRoleGetDcOperationResults
DsRoleGetPrimaryDomainInformation
DsRoleIfmHandleFree
DsRoleServerSaveStateForUpgrade
DsRoleUpgradeDownlevelServer
