;
; Definition file of XINPUT9_1_0.dll
; Automatic generated by gendef
; written by Kai Tietz 2008
;
LIBRARY "XINPUT9_1_0.dll"
EXPORTS
;DllMain
XInputGetCapabilities
XInputGetDSoundAudioDeviceGuids
XInputGetState
XInputSetState
