; 
; Exports of file ICAAPI.dll
;
; Autogenerated by gen_exportdef
; Written by Kai Tietz, 2007
;
LIBRARY ICAAPI.dll
EXPORTS
IcaCdCreateThread
IcaCdIoControl
IcaCdWaitForMultipleObjects
IcaCdWaitForSingleObject
IcaChannelClose
IcaChannelIoControl
IcaChannelOpen
IcaChannelTrace
IcaClose
IcaIoControl
IcaMemoryAllocate
IcaMemoryFree
IcaOpen
IcaPushConsoleStack
IcaStackCallback
IcaStackClose
IcaStackConnectionAccept
IcaStackConnectionClose
IcaStackConnectionRequest
IcaStackConnectionWait
IcaStackCreateShadowEndpoint
IcaStackDisconnect
IcaStackIoControl
IcaStackIoControlNoConnLock
IcaStackOpen
IcaStackQueryLocalAddress
IcaStackQueryState
IcaStackReconnect
IcaStackTerminate
IcaStackTrace
IcaStackUnlock
IcaSystemTrace
IcaTrace
_IcaStackIoControl
