LIBRARY coremessaging

EXPORTS

CoreUICallComputeMaximumMessageSize
CoreUICallCreateConversationHost
CoreUICallCreateEndpointHost
CoreUICallCreateEndpointHostWithSendPriority
CoreUICallGetAddressOfParameterInBuffer
CoreUICallReceive
CoreUICallSend
CoreUICallSendVaList
CoreUIConfigureTestHost@0
CoreUIConfigureUserIntegration@4
CoreUICreate@4
CoreUICreateAnonymousStream@4
CoreUICreateClientWindowIDManager@4
CoreUICreateEx@8
CoreUICreateSystemWindowIDManager@4
CoreUIInitializeTestService@12
CoreUIOpenExisting@4
CoreUIRouteToTestRegistrar@8
CoreUIUninitializeTestService@0
CreateDispatcherQueueController@16
CreateDispatcherQueueForCurrentThread@4
GetDispatcherQueueForCurrentThread@4
MsgBlobCreateShared@12
MsgBlobCreateStack@16
MsgBufferShare@8
MsgRelease@4
MsgStringCreateShared@12
MsgStringCreateStack@16
ServiceMain@8
