; 
; Exports of file URL.dll
;
; Autogenerated by gen_exportdef
; Written by Kai Tietz, 2007
;
LIBRARY URL.dll
EXPORTS
AddMIMEFileTypesPS
AutodialHookCallback
DllCanUnloadNow
DllGetClassObject
FileProtocolHandler
FileProtocolHandlerA
InetIsOffline
MIMEAssociationDialogA
MIMEAssociationDialogW
MailToProtocolHandler
MailToProtocolHandlerA
NewsProtocolHandler
NewsProtocolHandlerA
OpenURL
OpenURLA
TelnetProtocolHandler
TelnetProtocolHandlerA
TranslateURLA
TranslateURLW
URLAssociationDialogA
URLAssociationDialogW
