; 
; Exports of file SCROBJ.dll
;
; Autogenerated by gen_exportdef
; Written by Kai Tietz, 2007
;
LIBRARY SCROBJ.dll
EXPORTS
GenerateTypeLib
GenerateTypeLibW
DllCanUnloadNow
DllGetClassObject
DllInstall
DllRegisterServer
DllRegisterServerEx
DllRegisterServerExA
DllRegisterServerExW
DllUnregisterServer
DllUnregisterServerEx
