; 
; Exports of file schedsvc.dll
;
; Autogenerated by gen_exportdef
; Written by Kai Tietz, 2007
;
LIBRARY schedsvc.dll
EXPORTS
SPUninstall
SPUninstallCallback
SchedServiceMain
SysPrepBackup
SysPrepRestore
CloseProc
SysPrepCallback
