;
; Definition file of GPTEXT.DLL
; Automatic generated by gendef
; written by Kai Tietz 2008
;
LIBRARY "GPTEXT.DLL"
EXPORTS
ProcessEQoSPolicy@32
ProcessPSCHEDPolicy@32
DllRegisterServer
DllUnregisterServer
