;
; Definition file of fwpuclnt.dll
; Automatic generated by gendef
; written by Kai Tietz 2008
;
LIBRARY "fwpuclnt.dll"
EXPORTS
FwpiExpandCriteria0@20
FwpiFreeCriteria0@8
FwpiVpnTriggerAddAppSids@12
FwpiVpnTriggerAddFilePaths@12
FwpiVpnTriggerAddSecurityDescriptor@8
FwpiVpnTriggerConfigureParameters@8
FwpiVpnTriggerEventSubscribe0@20
FwpiVpnTriggerEventUnsubscribe0@8
FwpiVpnTriggerInitializeNrptTriggering@8
FwpiVpnTriggerRemoveAppSids@4
FwpiVpnTriggerRemoveFilePaths@4
FwpiVpnTriggerRemoveSecurityDescriptor@4
FwpiVpnTriggerResetNrptTriggering@4
FwpiVpnTriggerSetStateDisconnected@8
FwpiVpnTriggerUninitializeNrptTriggering@4
FwpmCalloutAdd0@16
FwpmCalloutCreateEnumHandle0@12
FwpmCalloutDeleteById0@8
FwpmCalloutDeleteByKey0@8
FwpmCalloutDestroyEnumHandle0@8
FwpmCalloutEnum0@20
FwpmCalloutGetById0@12
FwpmCalloutGetByKey0@12
FwpmCalloutGetSecurityInfoByKey0@32
FwpmCalloutSetSecurityInfoByKey0@28
FwpmCalloutSubscribeChanges0@20
FwpmCalloutSubscriptionsGet0@12
FwpmCalloutUnsubscribeChanges0@8
FwpmConnectionCreateEnumHandle0@12
FwpmConnectionDestroyEnumHandle0@8
FwpmConnectionEnum0@20
FwpmConnectionGetById0@16
FwpmConnectionGetSecurityInfo0@28
FwpmConnectionSetSecurityInfo0@24
FwpmConnectionSubscribe0@20
FwpmConnectionUnsubscribe0@8
FwpmDiagnoseNetFailure0@12
FwpmDynamicKeywordSubscribe0@16
FwpmDynamicKeywordUnsubscribe0@4
FwpmEngineClose0@4
FwpmEngineGetOption0@12
FwpmEngineGetSecurityInfo0@28
FwpmEngineOpen0@20
FwpmEngineSetOption0@12
FwpmEngineSetSecurityInfo0@24
FwpmEventProviderCreate0@8
FwpmEventProviderDestroy0@4
FwpmEventProviderFireNetEvent0@16
FwpmEventProviderFireNetEventEx0@20
FwpmEventProviderIsNetEventTypeEnabled0@12
FwpmFilterAdd0@16
FwpmFilterCreateEnumHandle0@12
FwpmFilterDeleteById0@12
FwpmFilterDeleteByKey0@8
FwpmFilterDestroyEnumHandle0@8
FwpmFilterEnum0@20
FwpmFilterGetById0@16
FwpmFilterGetByKey0@12
FwpmFilterGetSecurityInfoByKey0@32
FwpmFilterSetSecurityInfoByKey0@28
FwpmFilterSubscribeChanges0@20
FwpmFilterSubscriptionsGet0@12
FwpmFilterUnsubscribeChanges0@8
FwpmFreeMemory0@4
FwpmGetAppIdFromFileName0@8
FwpmGetSidFromOnlineId0@8
FwpmIPsecS2STunnelAddConditions0@28
FwpmIPsecS2STunnelAddInterfaceToCompartment0@12
FwpmIPsecS2STunnelGetInterfaceForCompartment0@12
FwpmIPsecS2STunnelRemoveConditions0@28
FwpmIPsecS2STunnelRemoveInterfaceFromCompartment0@8
FwpmIPsecTunnelAdd0@28
FwpmIPsecTunnelAdd1@32
FwpmIPsecTunnelAdd2@32
FwpmIPsecTunnelAdd3@32
FwpmIPsecTunnelAddConditions0@20
FwpmIPsecTunnelDeleteByKey0@8
FwpmLayerCreateEnumHandle0@12
FwpmLayerDestroyEnumHandle0@8
FwpmLayerEnum0@20
FwpmLayerGetById0@12
FwpmLayerGetByKey0@12
FwpmLayerGetSecurityInfoByKey0@32
FwpmLayerSetSecurityInfoByKey0@28
FwpmNetEventCreateEnumHandle0@12
FwpmNetEventCreateEnumHandleEx@16
FwpmNetEventDestroyEnumHandle0@8
FwpmNetEventEnum0@20
FwpmNetEventEnum1@20
FwpmNetEventEnum2@20
FwpmNetEventEnum3@20
FwpmNetEventEnum4@20
FwpmNetEventEnum5@20
FwpmNetEventSubscribe0@20
FwpmNetEventSubscribe1@20
FwpmNetEventSubscribe2@20
FwpmNetEventSubscribe3@20
FwpmNetEventSubscribe4@20
FwpmNetEventSubscriptionsGet0@12
FwpmNetEventUnsubscribe0@8
FwpmNetEventsGetSecurityInfo0@28
FwpmNetEventsLost0@8
FwpmNetEventsSetSecurityInfo0@24
FwpmProcessNameResolutionEvent0@16
FwpmProviderAdd0@12
FwpmProviderContextAdd0@16
FwpmProviderContextAdd1@16
FwpmProviderContextAdd2@16
FwpmProviderContextAdd3@16
FwpmProviderContextCreateEnumHandle0@12
FwpmProviderContextDeleteById0@12
FwpmProviderContextDeleteByKey0@8
FwpmProviderContextDestroyEnumHandle0@8
FwpmProviderContextEnum0@20
FwpmProviderContextEnum1@20
FwpmProviderContextEnum2@20
FwpmProviderContextEnum3@20
FwpmProviderContextGetById0@16
FwpmProviderContextGetById1@16
FwpmProviderContextGetById2@16
FwpmProviderContextGetById3@16
FwpmProviderContextGetByKey0@12
FwpmProviderContextGetByKey1@12
FwpmProviderContextGetByKey2@12
FwpmProviderContextGetByKey3@12
FwpmProviderContextGetSecurityInfoByKey0@32
FwpmProviderContextSetSecurityInfoByKey0@28
FwpmProviderContextSubscribeChanges0@20
FwpmProviderContextSubscriptionsGet0@12
FwpmProviderContextUnsubscribeChanges0@8
FwpmProviderCreateEnumHandle0@12
FwpmProviderDeleteByKey0@8
FwpmProviderDestroyEnumHandle0@8
FwpmProviderEnum0@20
FwpmProviderGetByKey0@12
FwpmProviderGetSecurityInfoByKey0@32
FwpmProviderSetSecurityInfoByKey0@28
FwpmProviderSubscribeChanges0@20
FwpmProviderSubscriptionsGet0@12
FwpmProviderUnsubscribeChanges0@8
FwpmSessionCreateEnumHandle0@12
FwpmSessionDestroyEnumHandle0@8
FwpmSessionEnum0@20
FwpmSubLayerAdd0@12
FwpmSubLayerCreateEnumHandle0@12
FwpmSubLayerDeleteByKey0@8
FwpmSubLayerDestroyEnumHandle0@8
FwpmSubLayerEnum0@20
FwpmSubLayerGetByKey0@12
FwpmSubLayerGetSecurityInfoByKey0@32
FwpmSubLayerSetSecurityInfoByKey0@28
FwpmSubLayerSubscribeChanges0@20
FwpmSubLayerSubscriptionsGet0@12
FwpmSubLayerUnsubscribeChanges0@8
FwpmSystemPortsGet0@8
FwpmSystemPortsSubscribe0@20
FwpmSystemPortsUnsubscribe0@8
FwpmTraceRestoreDefaults0@0
FwpmTransactionAbort0@4
FwpmTransactionBegin0@8
FwpmTransactionCommit0@4
FwpmvSwitchEventSubscribe0@20
FwpmvSwitchEventUnsubscribe0@8
FwpmvSwitchEventsGetSecurityInfo0@28
FwpmvSwitchEventsSetSecurityInfo0@24
FwppConnectionGetByIPsecInfo@20
FwppConnectionGetByS2STunnelId@16
FwppConnectionGetS2STunnelId@16
FwppGetMD5HashBytes@16
FwppIPsecSaContextCreate@36
FwpsAleEndpointCreateEnumHandle0@12
FwpsAleEndpointDestroyEnumHandle0@8
FwpsAleEndpointEnum0@20
FwpsAleEndpointGetById0@16
FwpsAleEndpointGetSecurityInfo0@28
FwpsAleEndpointSetSecurityInfo0@24
FwpsAleExplicitCredentialsQuery0@16
FwpsAleGetPortStatus0@12
FwpsClassifyUser0@28
FwpsFreeMemory0@4
FwpsGetInProcReplicaOffset0@4
FwpsLayerCreateInProcReplica0@8
FwpsLayerReleaseInProcReplica0@8
FwpsOpenToken0@20
FwpsQueryIPsecDosFWUsed0@8
FwpsQueryIPsecOffloadDone0@8
GetUnifiedTraceHandle@4
IPsecDospGetSecurityInfo0@28
IPsecDospGetStatistics0@8
IPsecDospSetSecurityInfo0@24
IPsecDospStateCreateEnumHandle0@12
IPsecDospStateDestroyEnumHandle0@8
IPsecDospStateEnum0@20
IPsecGetKeyFromDictator0@16
IPsecGetStatistics0@8
IPsecGetStatistics1@8
IPsecKeyDictationCheck0@16
IPsecKeyManagerAddAndRegister0@16
IPsecKeyManagerGetSecurityInfoByKey0@32
IPsecKeyManagerSetSecurityInfoByKey0@28
IPsecKeyManagerUnregisterAndDelete0@8
IPsecKeyManagersGet0@12
IPsecKeyModuleAdd0@12
IPsecKeyModuleDelete0@8
IPsecKeyModuleUpdateAcquire0@20
IPsecKeyNotification0@12
IPsecSaContextAddInbound0@16
IPsecSaContextAddInbound1@16
IPsecSaContextAddInboundAndTrackConnection@28
IPsecSaContextAddOutbound0@16
IPsecSaContextAddOutbound1@16
IPsecSaContextAddOutboundAndTrackConnection@28
IPsecSaContextCreate0@16
IPsecSaContextCreate1@20
IPsecSaContextCreateEnumHandle0@12
IPsecSaContextDeleteById0@12
IPsecSaContextDestroyEnumHandle0@8
IPsecSaContextEnum0@20
IPsecSaContextEnum1@20
IPsecSaContextExpire0@12
IPsecSaContextGetById0@16
IPsecSaContextGetById1@16
IPsecSaContextGetSpi0@20
IPsecSaContextGetSpi1@20
IPsecSaContextSetSpi0@20
IPsecSaContextSubscribe0@20
IPsecSaContextSubscriptionsGet0@12
IPsecSaContextUnsubscribe0@8
IPsecSaContextUpdate0@16
IPsecSaCreateEnumHandle0@12
IPsecSaDbGetSecurityInfo0@28
IPsecSaDbSetSecurityInfo0@24
IPsecSaDestroyEnumHandle0@8
IPsecSaEnum0@20
IPsecSaEnum1@20
IPsecSaInitiateAsync0@16
IkeextGetConfigParameters0@4
IkeextGetStatistics0@8
IkeextGetStatistics1@8
IkeextSaCreateEnumHandle0@12
IkeextSaDbGetSecurityInfo0@28
IkeextSaDbSetSecurityInfo0@24
IkeextSaDeleteById0@12
IkeextSaDestroyEnumHandle0@8
IkeextSaEnum0@20
IkeextSaEnum1@20
IkeextSaEnum2@20
IkeextSaGetById0@16
IkeextSaGetById1@20
IkeextSaGetById2@20
IkeextSaUpdateAdditionalAddressesByTunnelId0@24
IkeextSaUpdatePreferredAddressesByTunnelId0@24
IkeextSetConfigParameters0@4
NamespaceCallout@12
WFPRIODequeueCompletion@12
WSADeleteSocketPeerTargetName@20
WSAImpersonateSocketPeer@12
WSAQuerySocketSecurity@28
WSARevertImpersonation@0
WSASetSocketPeerTargetName@20
WSASetSocketSecurity@20
WfpCloseDPConfigureHandle@4 ; forwards to NtClose in ntdll.dll
WfpConfigureDPSecurityDescriptor@12
WfpCreateDPConfigureHandle@4
WfpRIOChannelClose@4 ; forwards to NtClose in ntdll.dll
WfpRIOCleanupRequestQueue@8
WfpRIOCloseCompletionQueue@8
WfpRIOCreateChannel@12
WfpRIOCreateCompletionQueue@16
WfpRIOCreateRequestQueue@36
WfpRIODeregisterBuffer@12
WfpRIOIndicateActivityThreshold@8
WfpRIONotify@8
WfpRIOReceive@28
WfpRIORegisterBuffer@16
WfpRIOResume@4
WfpRIOSend@28
WfpRIOSuspend@4
