; 
; Exports of file RASRAD.dll
;
; Autogenerated by gen_exportdef
; Written by Kai Tietz, 2007
;
LIBRARY RASRAD.dll
EXPORTS
Open
Collect
Close
RasAcctConfigChangeNotification
RasAcctProviderFreeAttributes
RasAcctProviderInitialize
RasAcctProviderInterimAccounting
RasAcctProviderStartAccounting
RasAcctProviderStopAccounting
RasAcctProviderTerminate
RasAuthConfigChangeNotification
RasAuthProviderAuthenticateUser
RasAuthProviderFreeAttributes
RasAuthProviderInitialize
RasAuthProviderTerminate
