; 
; Exports of file rasmans.dll
;
; Autogenerated by gen_exportdef
; Written by Kai Tietz, 2007
;
LIBRARY rasmans.dll
EXPORTS
ServiceMain
ServiceRequestInProcess
SetEntryDialParams
_RasmanEngine
_RasmanInit
