;
; Definition file of d3dx9_32.dll
; Automatic generated by gendef
; written by Kai Tietz 2008
;
LIBRARY "d3dx9_32.dll"
EXPORTS
D3DXAssembleShader@28
D3DXAssembleShaderFromFileA@24
D3DXAssembleShaderFromFileW@24
D3DXAssembleShaderFromResourceA@28
D3DXAssembleShaderFromResourceW@28
D3DXBoxBoundProbe@16
D3DXCheckCubeTextureRequirements@24
D3DXCheckTextureRequirements@28
D3DXCheckVersion@8
D3DXCheckVolumeTextureRequirements@32
D3DXCleanMesh@24
D3DXColorAdjustContrast@12
D3DXColorAdjustSaturation@12
D3DXCompileShader@40
D3DXCompileShaderFromFileA@36
D3DXCompileShaderFromFileW@36
D3DXCompileShaderFromResourceA@40
D3DXCompileShaderFromResourceW@40
D3DXComputeBoundingBox@20
D3DXComputeBoundingSphere@20
D3DXComputeIMTFromPerTexelSignal@44
D3DXComputeIMTFromPerVertexSignal@32
D3DXComputeIMTFromSignal@40
D3DXComputeIMTFromTexture@28
D3DXComputeNormalMap@24
D3DXComputeNormals@8
D3DXComputeTangent@24
D3DXComputeTangentFrame@8
D3DXComputeTangentFrameEx@64
D3DXConcatenateMeshes@32
D3DXConvertMeshSubsetToSingleStrip@20
D3DXConvertMeshSubsetToStrips@28
D3DXCreateAnimationController@20
D3DXCreateBox@24
D3DXCreateBuffer@8
D3DXCreateCompressedAnimationSet@32
D3DXCreateCubeTexture@28
D3DXCreateCubeTextureFromFileA@12
D3DXCreateCubeTextureFromFileExA@52
D3DXCreateCubeTextureFromFileExW@52
D3DXCreateCubeTextureFromFileInMemory@16
D3DXCreateCubeTextureFromFileInMemoryEx@56
D3DXCreateCubeTextureFromFileW@12
D3DXCreateCubeTextureFromResourceA@16
D3DXCreateCubeTextureFromResourceExA@56
D3DXCreateCubeTextureFromResourceExW@56
D3DXCreateCubeTextureFromResourceW@16
D3DXCreateCylinder@32
D3DXCreateEffect@36
D3DXCreateEffectCompiler@28
D3DXCreateEffectCompilerFromFileA@24
D3DXCreateEffectCompilerFromFileW@24
D3DXCreateEffectCompilerFromResourceA@28
D3DXCreateEffectCompilerFromResourceW@28
D3DXCreateEffectEx@40
D3DXCreateEffectFromFileA@32
D3DXCreateEffectFromFileExA@36
D3DXCreateEffectFromFileExW@36
D3DXCreateEffectFromFileW@32
D3DXCreateEffectFromResourceA@36
D3DXCreateEffectFromResourceExA@40
D3DXCreateEffectFromResourceExW@40
D3DXCreateEffectFromResourceW@36
D3DXCreateEffectPool@4
D3DXCreateFontA@48
D3DXCreateFontIndirectA@12
D3DXCreateFontIndirectW@12
D3DXCreateFontW@48
D3DXCreateFragmentLinker@12
D3DXCreateKeyframedAnimationSet@32
D3DXCreateLine@8
D3DXCreateMatrixStack@8
D3DXCreateMesh@24
D3DXCreateMeshFVF@24
D3DXCreateNPatchMesh@8
D3DXCreatePMeshFromStream@28
D3DXCreatePRTBuffer@16
D3DXCreatePRTBufferTex@20
D3DXCreatePRTCompBuffer@28
D3DXCreatePRTEngine@20
D3DXCreatePatchMesh@28
D3DXCreatePolygon@20
D3DXCreateRenderToEnvMap@28
D3DXCreateRenderToSurface@28
D3DXCreateSPMesh@20
D3DXCreateSkinInfo@16
D3DXCreateSkinInfoFVF@16
D3DXCreateSkinInfoFromBlendedMesh@16
D3DXCreateSphere@24
D3DXCreateSprite@8
D3DXCreateTeapot@12
D3DXCreateTextA@32
D3DXCreateTextW@32
D3DXCreateTexture@32
D3DXCreateTextureFromFileA@12
D3DXCreateTextureFromFileExA@56
D3DXCreateTextureFromFileExW@56
D3DXCreateTextureFromFileInMemory@16
D3DXCreateTextureFromFileInMemoryEx@60
D3DXCreateTextureFromFileW@12
D3DXCreateTextureFromResourceA@16
D3DXCreateTextureFromResourceExA@60
D3DXCreateTextureFromResourceExW@60
D3DXCreateTextureFromResourceW@16
D3DXCreateTextureGutterHelper@20
D3DXCreateTextureShader@8
D3DXCreateTorus@28
D3DXCreateVolumeTexture@36
D3DXCreateVolumeTextureFromFileA@12
D3DXCreateVolumeTextureFromFileExA@60
D3DXCreateVolumeTextureFromFileExW@60
D3DXCreateVolumeTextureFromFileInMemory@16
D3DXCreateVolumeTextureFromFileInMemoryEx@64
D3DXCreateVolumeTextureFromFileW@12
D3DXCreateVolumeTextureFromResourceA@16
D3DXCreateVolumeTextureFromResourceExA@64
D3DXCreateVolumeTextureFromResourceExW@64
D3DXCreateVolumeTextureFromResourceW@16
D3DXDebugMute@4
D3DXDeclaratorFromFVF@8
D3DXDisassembleEffect@12
D3DXDisassembleShader@16
D3DXFVFFromDeclarator@8
D3DXFileCreate@4
D3DXFillCubeTexture@12
D3DXFillCubeTextureTX@8
D3DXFillTexture@12
D3DXFillTextureTX@8
D3DXFillVolumeTexture@12
D3DXFillVolumeTextureTX@8
D3DXFilterTexture@16
D3DXFindShaderComment@16
D3DXFloat16To32Array@12
D3DXFloat32To16Array@12
D3DXFrameAppendChild@8
D3DXFrameCalculateBoundingSphere@12
D3DXFrameDestroy@8
D3DXFrameFind@8
D3DXFrameNumNamedMatrices@4
D3DXFrameRegisterNamedMatrices@8
D3DXFresnelTerm@8
D3DXGatherFragments@28
D3DXGatherFragmentsFromFileA@24
D3DXGatherFragmentsFromFileW@24
D3DXGatherFragmentsFromResourceA@28
D3DXGatherFragmentsFromResourceW@28
D3DXGenerateOutputDecl@8
D3DXGeneratePMesh@28
D3DXGetDeclLength@4
D3DXGetDeclVertexSize@8
D3DXGetDriverLevel@4
D3DXGetFVFVertexSize@4
D3DXGetImageInfoFromFileA@8
D3DXGetImageInfoFromFileInMemory@12
D3DXGetImageInfoFromFileW@8
D3DXGetImageInfoFromResourceA@12
D3DXGetImageInfoFromResourceW@12
D3DXGetPixelShaderProfile@4
D3DXGetShaderConstantTable@8
D3DXGetShaderInputSemantics@12
D3DXGetShaderOutputSemantics@12
D3DXGetShaderSamplers@12
D3DXGetShaderSize@4
D3DXGetShaderVersion@4
D3DXGetVertexShaderProfile@4
D3DXIntersect@40
D3DXIntersectSubset@44
D3DXIntersectTri@32
D3DXLoadMeshFromXA@32
D3DXLoadMeshFromXInMemory@36
D3DXLoadMeshFromXResource@40
D3DXLoadMeshFromXW@32
D3DXLoadMeshFromXof@32
D3DXLoadMeshHierarchyFromXA@28
D3DXLoadMeshHierarchyFromXInMemory@32
D3DXLoadMeshHierarchyFromXW@28
D3DXLoadPRTBufferFromFileA@8
D3DXLoadPRTBufferFromFileW@8
D3DXLoadPRTCompBufferFromFileA@8
D3DXLoadPRTCompBufferFromFileW@8
D3DXLoadPatchMeshFromXof@28
D3DXLoadSkinMeshFromXof@36
D3DXLoadSurfaceFromFileA@32
D3DXLoadSurfaceFromFileInMemory@36
D3DXLoadSurfaceFromFileW@32
D3DXLoadSurfaceFromMemory@40
D3DXLoadSurfaceFromResourceA@36
D3DXLoadSurfaceFromResourceW@36
D3DXLoadSurfaceFromSurface@32
D3DXLoadVolumeFromFileA@32
D3DXLoadVolumeFromFileInMemory@36
D3DXLoadVolumeFromFileW@32
D3DXLoadVolumeFromMemory@44
D3DXLoadVolumeFromResourceA@36
D3DXLoadVolumeFromResourceW@36
D3DXLoadVolumeFromVolume@32
D3DXMatrixAffineTransformation2D@20
D3DXMatrixAffineTransformation@20
D3DXMatrixDecompose@16
D3DXMatrixDeterminant@4
D3DXMatrixInverse@12
D3DXMatrixLookAtLH@16
D3DXMatrixLookAtRH@16
D3DXMatrixMultiply@12
D3DXMatrixMultiplyTranspose@12
D3DXMatrixOrthoLH@20
D3DXMatrixOrthoOffCenterLH@28
D3DXMatrixOrthoOffCenterRH@28
D3DXMatrixOrthoRH@20
D3DXMatrixPerspectiveFovLH@20
D3DXMatrixPerspectiveFovRH@20
D3DXMatrixPerspectiveLH@20
D3DXMatrixPerspectiveOffCenterLH@28
D3DXMatrixPerspectiveOffCenterRH@28
D3DXMatrixPerspectiveRH@20
D3DXMatrixReflect@8
D3DXMatrixRotationAxis@12
D3DXMatrixRotationQuaternion@8
D3DXMatrixRotationX@8
D3DXMatrixRotationY@8
D3DXMatrixRotationYawPitchRoll@16
D3DXMatrixRotationZ@8
D3DXMatrixScaling@16
D3DXMatrixShadow@12
D3DXMatrixTransformation2D@28
D3DXMatrixTransformation@28
D3DXMatrixTranslation@16
D3DXMatrixTranspose@8
D3DXOptimizeFaces@20
D3DXOptimizeVertices@20
D3DXPlaneFromPointNormal@12
D3DXPlaneFromPoints@16
D3DXPlaneIntersectLine@16
D3DXPlaneNormalize@8
D3DXPlaneTransform@12
D3DXPlaneTransformArray@24
D3DXPreprocessShader@24
D3DXPreprocessShaderFromFileA@20
D3DXPreprocessShaderFromFileW@20
D3DXPreprocessShaderFromResourceA@24
D3DXPreprocessShaderFromResourceW@24
D3DXQuaternionBaryCentric@24
D3DXQuaternionExp@8
D3DXQuaternionInverse@8
D3DXQuaternionLn@8
D3DXQuaternionMultiply@12
D3DXQuaternionNormalize@8
D3DXQuaternionRotationAxis@12
D3DXQuaternionRotationMatrix@8
D3DXQuaternionRotationYawPitchRoll@16
D3DXQuaternionSlerp@16
D3DXQuaternionSquad@24
D3DXQuaternionSquadSetup@28
D3DXQuaternionToAxisAngle@12
D3DXRectPatchSize@12
D3DXSHAdd@16
D3DXSHDot@12
D3DXSHEvalConeLight@36
D3DXSHEvalDirection@12
D3DXSHEvalDirectionalLight@32
D3DXSHEvalHemisphereLight@52
D3DXSHEvalSphericalLight@36
D3DXSHMultiply2@12
D3DXSHMultiply3@12
D3DXSHMultiply4@12
D3DXSHMultiply5@12
D3DXSHMultiply6@12
D3DXSHPRTCompSplitMeshSC@64
D3DXSHPRTCompSuperCluster@24
D3DXSHProjectCubeMap@20
D3DXSHRotate@16
D3DXSHRotateZ@16
D3DXSHScale@16
D3DXSaveMeshHierarchyToFileA@20
D3DXSaveMeshHierarchyToFileW@20
D3DXSaveMeshToXA@28
D3DXSaveMeshToXW@28
D3DXSavePRTBufferToFileA@8
D3DXSavePRTBufferToFileW@8
D3DXSavePRTCompBufferToFileA@8
D3DXSavePRTCompBufferToFileW@8
D3DXSaveSurfaceToFileA@20
D3DXSaveSurfaceToFileInMemory@20
D3DXSaveSurfaceToFileW@20
D3DXSaveTextureToFileA@16
D3DXSaveTextureToFileInMemory@16
D3DXSaveTextureToFileW@16
D3DXSaveVolumeToFileA@20
D3DXSaveVolumeToFileInMemory@20
D3DXSaveVolumeToFileW@20
D3DXSimplifyMesh@28
D3DXSphereBoundProbe@16
D3DXSplitMesh@36
D3DXTessellateNPatches@24
D3DXTessellateRectPatch@20
D3DXTessellateTriPatch@20
D3DXTriPatchSize@12
D3DXUVAtlasCreate@76
D3DXUVAtlasPack@44
D3DXUVAtlasPartition@68
D3DXValidMesh@12
D3DXValidPatchMesh@16
D3DXVec2BaryCentric@24
D3DXVec2CatmullRom@24
D3DXVec2Hermite@24
D3DXVec2Normalize@8
D3DXVec2Transform@12
D3DXVec2TransformArray@24
D3DXVec2TransformCoord@12
D3DXVec2TransformCoordArray@24
D3DXVec2TransformNormal@12
D3DXVec2TransformNormalArray@24
D3DXVec3BaryCentric@24
D3DXVec3CatmullRom@24
D3DXVec3Hermite@24
D3DXVec3Normalize@8
D3DXVec3Project@24
D3DXVec3ProjectArray@36
D3DXVec3Transform@12
D3DXVec3TransformArray@24
D3DXVec3TransformCoord@12
D3DXVec3TransformCoordArray@24
D3DXVec3TransformNormal@12
D3DXVec3TransformNormalArray@24
D3DXVec3Unproject@24
D3DXVec3UnprojectArray@36
D3DXVec4BaryCentric@24
D3DXVec4CatmullRom@24
D3DXVec4Cross@16
D3DXVec4Hermite@24
D3DXVec4Normalize@8
D3DXVec4Transform@12
D3DXVec4TransformArray@24
D3DXWeldVertices@28
