;
; Definition file of AppxDeploymentExtensions.dll
; Automatic generated by gendef
; written by Kai Tietz 2008-2014
;
LIBRARY "AppxDeploymentExtensions.dll"
EXPORTS
LoadCategoryNameTable
LoadExtensionRegistrationTable
ShellRefresh
