;
; Definition file of winsrpc.dll
; Automatic generated by gendef
; written by Kai Tietz 2008-2014
;
LIBRARY "winsrpc.dll"
EXPORTS
WinsABind
WinsAllocMem
WinsBackup
WinsCheckAccess
WinsDelDbRecs
WinsDeleteWins
WinsDoScavenging
WinsDoScavengingNew
WinsDoStaticInit
WinsFreeMem
WinsGetBrowserNames
WinsGetDbRecs
WinsGetDbRecsByName
WinsGetNameAndAdd
WinsPullRange
WinsRecordAction
WinsResetCounters
WinsRestore
WinsRestoreEx
WinsSetFlags
WinsSetPriorityClass
WinsStatus
WinsStatusNew
WinsStatusWHdl
WinsSyncUp
WinsTerm
WinsTombstoneDbRecs
WinsTrigger
WinsUBind
WinsUnbind
WinsWorkerThdUpd
