LIBRARY snmpapi.dll
EXPORTS
SnmpSvcAddrIsIpx@12
SnmpSvcAddrToSocket@8
SnmpSvcBufRevAndCpy@12
SnmpSvcBufRevInPlace@8
SnmpSvcDecodeMessage@20
SnmpSvcEncodeMessage@16
SnmpSvcGenerateAuthFailTrap@4
SnmpSvcGenerateColdStartTrap@4
SnmpSvcGenerateLinkDownTrap@8
SnmpSvcGenerateLinkUpTrap@8
SnmpSvcGenerateTrap@20
SnmpSvcGenerateWarmStartTrap@4
SnmpSvcGetUptime@0
SnmpSvcInitUptime@0
SnmpSvcReleaseMessage@4
SnmpSvcReportEvent@16
SnmpSvcSetLogLevel@4
SnmpSvcSetLogType@4
SnmpUtilAnsiToUnicode@12
SnmpUtilDbgPrint
SnmpUtilIdsToA@8
SnmpUtilMemAlloc@4
SnmpUtilMemFree@4
SnmpUtilMemReAlloc@8
SnmpUtilOidAppend@8
SnmpUtilOidCmp@8
SnmpUtilOidCpy@8
SnmpUtilOidFree@4
SnmpUtilOidNCmp@12
SnmpUtilOidToA@4
SnmpUtilPrintAsnAny@4
SnmpUtilPrintOid@4
SnmpUtilStrlenW@4
SnmpUtilUnicodeToAnsi@12
SnmpUtilVarBindCpy@8
SnmpUtilVarBindFree@4
SnmpUtilVarBindListCpy@8
SnmpUtilVarBindListFree@4
