/* oaidl-uuid.c */
/* Generate GUIDs for OA IDL interfaces */

/* All IIDs defined in this file were extracted from
 * HKEY_LOCAL_MACHINE\SOFTWARE\Classes\Interface\ */

#define INITGUID
#include <basetyps.h>
DEFINE_GUID(IID_IErrorInfo,0x1cf2b120,0x547d,0x101b,0x8e,0x65,0x8,0,0x2b,0x2b,0xd1,0x19);
DEFINE_GUID(IID_ICreateErrorInfo,0x22f03340,0x547d,0x101b,0x8e,0x65,0x8,0,0x2b,0x2b,0xd1,0x19);
DEFINE_GUID(IID_ISupportErrorInfo,0xdf0b3d60,0x548f,0x101b,0x8e,0x65,0x8,0,0x2b,0x2b,0xd1,0x19);
DEFINE_OLEGUID(IID_ICreateTypeInfo,0x20405,0,0);
DEFINE_OLEGUID(IID_ICreateTypeInfo2,0x2040e,0,0);
DEFINE_OLEGUID(IID_ICreateTypeLib,0x20406,0,0);
DEFINE_OLEGUID(IID_ICreateTypeLib2,0x2040F,0,0);
DEFINE_OLEGUID(IID_ITypeInfo,0x20401,0,0);
DEFINE_OLEGUID(IID_ITypeInfo2,0x20412,0,0);
DEFINE_OLEGUID(IID_ITypeLib,0x20402,0,0);
DEFINE_OLEGUID(IID_ITypeLib2,0x20411,0,0);
DEFINE_OLEGUID(IID_IEnumVARIANT,0x20404,0,0);
