;
; Definition file of XINPUT1_3.dll
; Automatic generated by gendef
; written by Kai Tietz 2008
;
LIBRARY "XINPUT1_3.dll"
EXPORTS
XInputGetState
XInputSetState
XInputGetCapabilities
XInputEnable
XInputGetDSoundAudioDeviceGuids
XInputGetBatteryInformation
XInputGetKeystroke
;ord_100 @100
;ord_101 @101
;ord_102 @102
;ord_103 @103
