;
; Definition file of CLUSAPI.dll
; Automatic generated by gendef
; written by Kai Tietz 2008
;
LIBRARY "CLUSAPI.dll"
EXPORTS
AddClusterNode@16
AddClusterResourceDependency@8
AddClusterResourceNode@8
BackupClusterDatabase@8
CanResourceBeDependent@8
ChangeClusterResourceGroup@8
CloseCluster@4
CloseClusterGroup@4
CloseClusterNetInterface@4
CloseClusterNetwork@4
CloseClusterNode@4
CloseClusterNotifyPort@4
CloseClusterResource@4
ClusterCloseEnum@4
ClusterControl@32
ClusterEnum@20
ClusterGetEnumCount@4
ClusterGroupCloseEnum@4
ClusterGroupControl@32
ClusterGroupEnum@20
ClusterGroupGetEnumCount@4
ClusterGroupOpenEnum@8
ClusterNetInterfaceControl@32
ClusterNetworkCloseEnum@4
ClusterNetworkControl@32
ClusterNetworkEnum@20
ClusterNetworkGetEnumCount@4
ClusterNetworkOpenEnum@8
ClusterNodeCloseEnum@4
ClusterNodeControl@32
ClusterNodeEnum@20
ClusterNodeGetEnumCount@4
ClusterNodeOpenEnum@8
ClusterOpenEnum@8
ClusterRegBatchAddCommand@24
ClusterRegBatchCloseNotification@4
ClusterRegBatchReadCommand@8
ClusterRegCloseBatch@12
ClusterRegCloseBatchNotifyPort@4
ClusterRegCloseKey@4
ClusterRegCreateBatch@8
ClusterRegCreateBatchNotifyPort@8
ClusterRegCreateKey@28
ClusterRegDeleteKey@8
ClusterRegDeleteValue@8
ClusterRegEnumKey@20
ClusterRegEnumValue@28
ClusterRegGetBatchNotification@8
ClusterRegGetKeySecurity@16
ClusterRegOpenKey@16
ClusterRegQueryInfoKey@32
ClusterRegQueryValue@20
ClusterRegSetKeySecurity@12
ClusterRegSetValue@20
ClusterResourceCloseEnum@4
ClusterResourceControl@32
ClusterResourceEnum@20
ClusterResourceGetEnumCount@4
ClusterResourceOpenEnum@8
ClusterResourceTypeCloseEnum@4
ClusterResourceTypeControl@36
ClusterResourceTypeEnum@20
ClusterResourceTypeGetEnumCount@4
ClusterResourceTypeOpenEnum@12
CreateCluster@12
CreateClusterGroup@8
CreateClusterNotifyPort@16
CreateClusterResource@16
CreateClusterResourceType@24
DeleteClusterGroup@4
DeleteClusterResource@4
DeleteClusterResourceType@8
DestroyCluster@16
DestroyClusterGroup@4
EvictClusterNode@4
EvictClusterNodeEx@12
FailClusterResource@4
GetClusterFromGroup@4
GetClusterFromNetInterface@4
GetClusterFromNetwork@4
GetClusterFromNode@4
GetClusterFromResource@4
GetClusterGroupKey@8
GetClusterGroupState@12
GetClusterInformation@16
GetClusterKey@8
GetClusterNetInterface@20
GetClusterNetInterfaceKey@8
GetClusterNetInterfaceState@4
GetClusterNetworkId@12
GetClusterNetworkKey@8
GetClusterNetworkState@4
GetClusterNodeId@12
GetClusterNodeKey@8
GetClusterNodeState@4
GetClusterNotify@24
GetClusterQuorumResource@24
GetClusterResourceDependencyExpression@12
GetClusterResourceKey@8
GetClusterResourceNetworkName@12
GetClusterResourceState@20
GetClusterResourceTypeKey@12
GetNodeClusterState@8
MoveClusterGroup@8
OfflineClusterGroup@4
OfflineClusterResource@4
OnlineClusterGroup@8
OnlineClusterResource@4
OpenCluster@4
OpenClusterGroup@8
OpenClusterNetInterface@8
OpenClusterNetwork@8
OpenClusterNode@8
OpenClusterResource@8
PauseClusterNode@4
RegisterClusterNotify@16
RemoveClusterResourceDependency@8
RemoveClusterResourceNode@8
RestoreClusterDatabase@12
ResumeClusterNode@4
SetClusterGroupName@8
SetClusterGroupNodeList@12
SetClusterName@8
SetClusterNetworkName@8
SetClusterNetworkPriorityOrder@12
SetClusterQuorumResource@12
SetClusterResourceDependencyExpression@8
SetClusterResourceName@8
SetClusterServiceAccountPassword@20
