#ifndef _ASM_UNISTD_32_H
#define _ASM_UNISTD_32_H

#define __NR_spill 0
#define __NR_xtensa 1
#define __NR_available4 2
#define __NR_available5 3
#define __NR_available6 4
#define __NR_available7 5
#define __NR_available8 6
#define __NR_available9 7
#define __NR_open 8
#define __NR_close 9
#define __NR_dup 10
#define __NR_dup2 11
#define __NR_read 12
#define __NR_write 13
#define __NR_select 14
#define __NR_lseek 15
#define __NR_poll 16
#define __NR__llseek 17
#define __NR_epoll_wait 18
#define __NR_epoll_ctl 19
#define __NR_epoll_create 20
#define __NR_creat 21
#define __NR_truncate 22
#define __NR_ftruncate 23
#define __NR_readv 24
#define __NR_writev 25
#define __NR_fsync 26
#define __NR_fdatasync 27
#define __NR_truncate64 28
#define __NR_ftruncate64 29
#define __NR_pread64 30
#define __NR_pwrite64 31
#define __NR_link 32
#define __NR_rename 33
#define __NR_symlink 34
#define __NR_readlink 35
#define __NR_mknod 36
#define __NR_pipe 37
#define __NR_unlink 38
#define __NR_rmdir 39
#define __NR_mkdir 40
#define __NR_chdir 41
#define __NR_fchdir 42
#define __NR_getcwd 43
#define __NR_chmod 44
#define __NR_chown 45
#define __NR_stat 46
#define __NR_stat64 47
#define __NR_lchown 48
#define __NR_lstat 49
#define __NR_lstat64 50
#define __NR_available51 51
#define __NR_fchmod 52
#define __NR_fchown 53
#define __NR_fstat 54
#define __NR_fstat64 55
#define __NR_flock 56
#define __NR_access 57
#define __NR_umask 58
#define __NR_getdents 59
#define __NR_getdents64 60
#define __NR_fcntl64 61
#define __NR_fallocate 62
#define __NR_fadvise64_64 63
#define __NR_utime 64
#define __NR_utimes 65
#define __NR_ioctl 66
#define __NR_fcntl 67
#define __NR_setxattr 68
#define __NR_getxattr 69
#define __NR_listxattr 70
#define __NR_removexattr 71
#define __NR_lsetxattr 72
#define __NR_lgetxattr 73
#define __NR_llistxattr 74
#define __NR_lremovexattr 75
#define __NR_fsetxattr 76
#define __NR_fgetxattr 77
#define __NR_flistxattr 78
#define __NR_fremovexattr 79
#define __NR_mmap2 80
#define __NR_munmap 81
#define __NR_mprotect 82
#define __NR_brk 83
#define __NR_mlock 84
#define __NR_munlock 85
#define __NR_mlockall 86
#define __NR_munlockall 87
#define __NR_mremap 88
#define __NR_msync 89
#define __NR_mincore 90
#define __NR_madvise 91
#define __NR_shmget 92
#define __NR_shmat 93
#define __NR_shmctl 94
#define __NR_shmdt 95
#define __NR_socket 96
#define __NR_setsockopt 97
#define __NR_getsockopt 98
#define __NR_shutdown 99
#define __NR_bind 100
#define __NR_connect 101
#define __NR_listen 102
#define __NR_accept 103
#define __NR_getsockname 104
#define __NR_getpeername 105
#define __NR_sendmsg 106
#define __NR_recvmsg 107
#define __NR_send 108
#define __NR_recv 109
#define __NR_sendto 110
#define __NR_recvfrom 111
#define __NR_socketpair 112
#define __NR_sendfile 113
#define __NR_sendfile64 114
#define __NR_sendmmsg 115
#define __NR_clone 116
#define __NR_execve 117
#define __NR_exit 118
#define __NR_exit_group 119
#define __NR_getpid 120
#define __NR_wait4 121
#define __NR_waitid 122
#define __NR_kill 123
#define __NR_tkill 124
#define __NR_tgkill 125
#define __NR_set_tid_address 126
#define __NR_gettid 127
#define __NR_setsid 128
#define __NR_getsid 129
#define __NR_prctl 130
#define __NR_personality 131
#define __NR_getpriority 132
#define __NR_setpriority 133
#define __NR_setitimer 134
#define __NR_getitimer 135
#define __NR_setuid 136
#define __NR_getuid 137
#define __NR_setgid 138
#define __NR_getgid 139
#define __NR_geteuid 140
#define __NR_getegid 141
#define __NR_setreuid 142
#define __NR_setregid 143
#define __NR_setresuid 144
#define __NR_getresuid 145
#define __NR_setresgid 146
#define __NR_getresgid 147
#define __NR_setpgid 148
#define __NR_getpgid 149
#define __NR_getppid 150
#define __NR_getpgrp 151
#define __NR_reserved152 152
#define __NR_reserved153 153
#define __NR_times 154
#define __NR_acct 155
#define __NR_sched_setaffinity 156
#define __NR_sched_getaffinity 157
#define __NR_capget 158
#define __NR_capset 159
#define __NR_ptrace 160
#define __NR_semtimedop 161
#define __NR_semget 162
#define __NR_semop 163
#define __NR_semctl 164
#define __NR_available165 165
#define __NR_msgget 166
#define __NR_msgsnd 167
#define __NR_msgrcv 168
#define __NR_msgctl 169
#define __NR_available170 170
#define __NR_umount2 171
#define __NR_mount 172
#define __NR_swapon 173
#define __NR_chroot 174
#define __NR_pivot_root 175
#define __NR_umount 176
#define __NR_swapoff 177
#define __NR_sync 178
#define __NR_syncfs 179
#define __NR_setfsuid 180
#define __NR_setfsgid 181
#define __NR_sysfs 182
#define __NR_ustat 183
#define __NR_statfs 184
#define __NR_fstatfs 185
#define __NR_statfs64 186
#define __NR_fstatfs64 187
#define __NR_setrlimit 188
#define __NR_getrlimit 189
#define __NR_getrusage 190
#define __NR_futex 191
#define __NR_gettimeofday 192
#define __NR_settimeofday 193
#define __NR_adjtimex 194
#define __NR_nanosleep 195
#define __NR_getgroups 196
#define __NR_setgroups 197
#define __NR_sethostname 198
#define __NR_setdomainname 199
#define __NR_syslog 200
#define __NR_vhangup 201
#define __NR_uselib 202
#define __NR_reboot 203
#define __NR_quotactl 204
#define __NR_nfsservctl 205
#define __NR__sysctl 206
#define __NR_bdflush 207
#define __NR_uname 208
#define __NR_sysinfo 209
#define __NR_init_module 210
#define __NR_delete_module 211
#define __NR_sched_setparam 212
#define __NR_sched_getparam 213
#define __NR_sched_setscheduler 214
#define __NR_sched_getscheduler 215
#define __NR_sched_get_priority_max 216
#define __NR_sched_get_priority_min 217
#define __NR_sched_rr_get_interval 218
#define __NR_sched_yield 219
#define __NR_available222 222
#define __NR_restart_syscall 223
#define __NR_sigaltstack 224
#define __NR_rt_sigreturn 225
#define __NR_rt_sigaction 226
#define __NR_rt_sigprocmask 227
#define __NR_rt_sigpending 228
#define __NR_rt_sigtimedwait 229
#define __NR_rt_sigqueueinfo 230
#define __NR_rt_sigsuspend 231
#define __NR_mq_open 232
#define __NR_mq_unlink 233
#define __NR_mq_timedsend 234
#define __NR_mq_timedreceive 235
#define __NR_mq_notify 236
#define __NR_mq_getsetattr 237
#define __NR_available238 238
#define __NR_io_setup 239
#define __NR_io_destroy 240
#define __NR_io_submit 241
#define __NR_io_getevents 242
#define __NR_io_cancel 243
#define __NR_clock_settime 244
#define __NR_clock_gettime 245
#define __NR_clock_getres 246
#define __NR_clock_nanosleep 247
#define __NR_timer_create 248
#define __NR_timer_delete 249
#define __NR_timer_settime 250
#define __NR_timer_gettime 251
#define __NR_timer_getoverrun 252
#define __NR_reserved253 253
#define __NR_lookup_dcookie 254
#define __NR_available255 255
#define __NR_add_key 256
#define __NR_request_key 257
#define __NR_keyctl 258
#define __NR_available259 259
#define __NR_readahead 260
#define __NR_remap_file_pages 261
#define __NR_migrate_pages 262
#define __NR_mbind 263
#define __NR_get_mempolicy 264
#define __NR_set_mempolicy 265
#define __NR_unshare 266
#define __NR_move_pages 267
#define __NR_splice 268
#define __NR_tee 269
#define __NR_vmsplice 270
#define __NR_available271 271
#define __NR_pselect6 272
#define __NR_ppoll 273
#define __NR_epoll_pwait 274
#define __NR_epoll_create1 275
#define __NR_inotify_init 276
#define __NR_inotify_add_watch 277
#define __NR_inotify_rm_watch 278
#define __NR_inotify_init1 279
#define __NR_getcpu 280
#define __NR_kexec_load 281
#define __NR_ioprio_set 282
#define __NR_ioprio_get 283
#define __NR_set_robust_list 284
#define __NR_get_robust_list 285
#define __NR_available286 286
#define __NR_available287 287
#define __NR_openat 288
#define __NR_mkdirat 289
#define __NR_mknodat 290
#define __NR_unlinkat 291
#define __NR_renameat 292
#define __NR_linkat 293
#define __NR_symlinkat 294
#define __NR_readlinkat 295
#define __NR_utimensat 296
#define __NR_fchownat 297
#define __NR_futimesat 298
#define __NR_fstatat64 299
#define __NR_fchmodat 300
#define __NR_faccessat 301
#define __NR_available302 302
#define __NR_available303 303
#define __NR_signalfd 304
#define __NR_eventfd 306
#define __NR_recvmmsg 307
#define __NR_setns 308
#define __NR_signalfd4 309
#define __NR_dup3 310
#define __NR_pipe2 311
#define __NR_timerfd_create 312
#define __NR_timerfd_settime 313
#define __NR_timerfd_gettime 314
#define __NR_available315 315
#define __NR_eventfd2 316
#define __NR_preadv 317
#define __NR_pwritev 318
#define __NR_available319 319
#define __NR_fanotify_init 320
#define __NR_fanotify_mark 321
#define __NR_process_vm_readv 322
#define __NR_process_vm_writev 323
#define __NR_name_to_handle_at 324
#define __NR_open_by_handle_at 325
#define __NR_sync_file_range2 326
#define __NR_perf_event_open 327
#define __NR_rt_tgsigqueueinfo 328
#define __NR_clock_adjtime 329
#define __NR_prlimit64 330
#define __NR_kcmp 331
#define __NR_finit_module 332
#define __NR_accept4 333
#define __NR_sched_setattr 334
#define __NR_sched_getattr 335
#define __NR_renameat2 336
#define __NR_seccomp 337
#define __NR_getrandom 338
#define __NR_memfd_create 339
#define __NR_bpf 340
#define __NR_execveat 341
#define __NR_userfaultfd 342
#define __NR_membarrier 343
#define __NR_mlock2 344
#define __NR_copy_file_range 345
#define __NR_preadv2 346
#define __NR_pwritev2 347
#define __NR_pkey_mprotect 348
#define __NR_pkey_alloc 349
#define __NR_pkey_free 350
#define __NR_statx 351
#define __NR_rseq 352
#define __NR_clock_gettime64 403
#define __NR_clock_settime64 404
#define __NR_clock_adjtime64 405
#define __NR_clock_getres_time64 406
#define __NR_clock_nanosleep_time64 407
#define __NR_timer_gettime64 408
#define __NR_timer_settime64 409
#define __NR_timerfd_gettime64 410
#define __NR_timerfd_settime64 411
#define __NR_utimensat_time64 412
#define __NR_pselect6_time64 413
#define __NR_ppoll_time64 414
#define __NR_io_pgetevents_time64 416
#define __NR_recvmmsg_time64 417
#define __NR_mq_timedsend_time64 418
#define __NR_mq_timedreceive_time64 419
#define __NR_semtimedop_time64 420
#define __NR_rt_sigtimedwait_time64 421
#define __NR_futex_time64 422
#define __NR_sched_rr_get_interval_time64 423
#define __NR_pidfd_send_signal 424
#define __NR_io_uring_setup 425
#define __NR_io_uring_enter 426
#define __NR_io_uring_register 427
#define __NR_open_tree 428
#define __NR_move_mount 429
#define __NR_fsopen 430
#define __NR_fsconfig 431
#define __NR_fsmount 432
#define __NR_fspick 433
#define __NR_pidfd_open 434
#define __NR_clone3 435
#define __NR_close_range 436
#define __NR_openat2 437
#define __NR_pidfd_getfd 438
#define __NR_faccessat2 439
#define __NR_process_madvise 440
#define __NR_epoll_pwait2 441
#define __NR_mount_setattr 442
#define __NR_quotactl_fd 443
#define __NR_landlock_create_ruleset 444
#define __NR_landlock_add_rule 445
#define __NR_landlock_restrict_self 446
#define __NR_process_mrelease 448
#define __NR_futex_waitv 449
#define __NR_set_mempolicy_home_node 450
#define __NR_cachestat 451
#define __NR_fchmodat2 452
#define __NR_map_shadow_stack 453
#define __NR_futex_wake 454
#define __NR_futex_wait 455
#define __NR_futex_requeue 456
#define __NR_statmount 457
#define __NR_listmount 458
#define __NR_lsm_get_self_attr 459
#define __NR_lsm_set_self_attr 460
#define __NR_lsm_list_modules 461
#define __NR_mseal 462
#define __NR_setxattrat 463
#define __NR_getxattrat 464
#define __NR_listxattrat 465
#define __NR_removexattrat 466
#define __NR_open_tree_attr 467
#define __NR_file_getattr 468
#define __NR_file_setattr 469
#define __NR_listns 470


#endif /* _ASM_UNISTD_32_H */