; 
; Exports of file ICFGNT.dll
;
; Autogenerated by gen_exportdef
; Written by Kai Tietz, 2007
;
LIBRARY ICFGNT.dll
EXPORTS
IcfgSetInstallSourcePath
InetSetAutodialAddress
IcfgGetLastInstallErrorText
IcfgInstallInetComponents
IcfgInstallModem
IcfgIsFileSharingTurnedOn
IcfgIsGlobalDNS
IcfgNeedInetComponents
IcfgNeedModem
IcfgRemoveGlobalDNS
IcfgStartServices
IcfgTurnOffFileSharing
InetGetAutodial
InetGetSupportedPlatform
InetSetAutodial
