LIBRARY api-ms-win-eventing-legacy-l1-1-0

EXPORTS

ControlTraceA
EnableTrace
EnableTraceEx
EnumerateTraceGuids
FlushTraceA
FlushTraceW
OpenTraceA
QueryAllTracesA
QueryTraceA
QueryTraceW
StartTraceA
StopTraceA
UpdateTraceA
UpdateTraceW
