;
; Definition file of certca.dll
; Automatic generated by gendef
; written by Kai Tietz 2008-2014
;
LIBRARY "certca.dll"
EXPORTS
CAFindByName
CAFindByCertType
CAFindByIssuerDN
CAEnumFirstCA
CAEnumNextCA
CACreateNewCA
CAUpdateCA
CAUpdateCAEx
CADeleteCA
CADeleteCAEx
CACountCAs
CACloseCA
CAGetCAProperty
CAFreeCAProperty
CASetCAProperty
CAGetCACertificate
CASetCACertificate
CAGetCAExpiration
CASetCAExpiration
CAGetCASecurity
CASetCASecurity
CAAccessCheck
CAAccessCheckEx
CAEnumCertTypesForCA
CAEnumCertTypesForCAEx
CAAddCACertificateType
CAAddCACertificateTypeEx
CARemoveCACertificateType
CARemoveCACertificateTypeEx
CAGetCAFlags
CASetCAFlags
CAGetDN
CAEnumCertTypes
CAEnumCertTypesEx
CAFindCertTypeByName
CACreateCertType
CAUpdateCertType
CAUpdateCertTypeEx
CADeleteCertType
CADeleteCertTypeEx
CACloneCertType
CAEnumNextCertType
CACountCertTypes
CACloseCertType
CAGetCertTypeProperty
CAGetCertTypePropertyEx
CASetCertTypeProperty
CASetCertTypePropertyEx
CADCSetCertTypePropertyEx
CAFreeCertTypeProperty
CAGetCertTypeExtensions
CAGetCertTypeExtensionsEx
CAFreeCertTypeExtensions
CASetCertTypeExtension
CAGetCertTypeKeySpec
CASetCertTypeKeySpec
CAGetCertTypeExpiration
CASetCertTypeExpiration
CAGetCertTypeFlags
CAGetCertTypeFlagsEx
CASetCertTypeFlags
CASetCertTypeFlagsEx
CAInstallDefaultCertType
CAInstallDefaultCertTypeEx
CAIsCertTypeCurrent
CAIsCertTypeCurrentEx
CACertTypeGetSecurity
CACertTypeSetSecurity
CACertTypeAccessCheck
CACertTypeAccessCheckEx
CACertTypeAuthzAccessCheck
CAOIDCreateNew
CAOIDCreateNewEx
CAOIDSetProperty
CAOIDSetPropertyEx
CAOIDAdd
CAOIDAddEx
CAOIDDelete
CAOIDDeleteEx
CAOIDGetProperty
CAOIDGetPropertyEx
CAOIDFreeProperty
CAOIDGetLdapURL
CAOIDFreeLdapURL
CACertTypeRegisterQuery
CACertTypeQuery
CACertTypeUnregisterQuery
CACreateLocalAutoEnrollmentObject
CADeleteLocalAutoEnrollmentObject
CACreateAutoEnrollmentObjectEx
CAEnumCertTypesEx2
CAFindCertTypeByName2
ord_601 @601
ord_602 @602
ord_603 @603
ord_604 @604
ord_701 @701
ord_702 @702
ord_703 @703
ord_704 @704
ord_705 @705
ord_706 @706
ord_707 @707
ord_708 @708
ord_801 @801
ord_802 @802
ord_803 @803
ord_804 @804
ord_805 @805
ord_806 @806
ord_807 @807
ord_808 @808
ord_809 @809
ord_810 @810
ord_811 @811
ord_812 @812
ord_813 @813
ord_814 @814
ord_815 @815
ord_816 @816
ord_817 @817
ord_818 @818
ord_819 @819
ord_820 @820
ord_821 @821
ord_822 @822
ord_823 @823
ord_824 @824
ord_825 @825
ord_826 @826
ord_827 @827
ord_828 @828
ord_829 @829
ord_830 @830
ord_831 @831
ord_832 @832
ord_833 @833
ord_834 @834
ord_835 @835
ord_836 @836
ord_837 @837
ord_838 @838
ord_839 @839
ord_840 @840
ord_841 @841
ord_842 @842
ord_843 @843
ord_844 @844
ord_845 @845
ord_846 @846
ord_847 @847
ord_848 @848
ord_849 @849
ord_850 @850
ord_851 @851
ord_852 @852
ord_853 @853
ord_854 @854
ord_855 @855
ord_856 @856
ord_857 @857
ord_858 @858
ord_859 @859
CCFindCertificateBuildFilter
CCFindCertificateFreeFilter
CCFindCertificateFromFilter
CCGetCertNameList
CCFreeStringArray
ord_865 @865
ord_866 @866
ord_867 @867
ord_868 @868
ord_869 @869
ord_870 @870
ord_871 @871
ord_872 @872
