;
; Definition file of WDSCSL.dll
; Automatic generated by gendef
; written by Kai Tietz 2008
;
LIBRARY "WDSCSL.dll"
EXPORTS
WdsClientExecute@32
WdsClientInitializeLibrary
WdsClientPacketAllocate@4
WdsClientPacketFree@4
WdsClientRegisterTrace@4
WdsClientSessionCreate@16
WdsClientSessionExecute@28
WdsClientSessionShutdown@4
WdsCpPacketGetBuffer@12
WdsCpPacketInitialize@12
WdsCpPacketRelease@4
WdsCpParameterAdd@20
WdsCpParameterDelete@8
WdsCpParameterQuery@24
WdsCpParameterValidate@28
WdsCpRecvPacketInitialize@20
